#!/bin/sh

#  Copyright (c) Members of the EGEE Collaboration. 2004. 
#  See http://www.eu-egee.org/partners/ for details on the copyright
#  holders.  
#  
#  Licensed under the Apache License, Version 2.0 (the "License"); 
#  you may not use this file except in compliance with the License. 
#  You may obtain a copy of the License at 
#  
#      http://www.apache.org/licenses/LICENSE-2.0 
#  
#  Unless required by applicable law or agreed to in writing, software 
#  distributed under the License is distributed on an "AS IS" BASIS, 
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
#  See the License for the specific language governing permissions and 
#  limitations under the License.

   if [ $# -ne 7 ]; then
         echo 1>&2 Usage: $0 userDB pswDB DBName creamDBVersion creationDBScript hostDB creamSandboxPath
         exit -1
    fi

dbVersionMustBe="$4"
echo  The database version requested by cream service is $dbVersionMustBe
creationDBScript="$5"
hostDB="$6"
creamSandboxPath="$7"

dbVersionFromDb=`mysql -N -B --host $hostDB --user="$1" --password="$2" -D "$3" -e "select version from db_info;" 2>&1`
exitCode=$?
if [ "$exitCode" -ne 0 ]
then 
  echo "Impossible to retrieve the version of $3 database. Database will be created from scratch."
  dbVersionFromDb=0
else
  echo "The version retrieved from the $3 database is $dbVersionFromDb"
fi

if [ "$exitCode" -ne 0 -o X"$dbVersionMustBe" != X"$dbVersionFromDb" ]
then
  echo "Creating/Updating $3 database..."
  mysql --host $hostDB --user="$1" --password="$2" < "$creationDBScript"
  exitCode=$?
  if [  "$exitCode" -ne 0 ]
  then
    echo An error occurred on the creation of the database
    exit -1
  fi
  echo "$3 database created!"
  if [ $3 == "creamdb" -a -d $creamSandboxPath ] ; then
    # remove cream_sandbox
    echo "Removing cream_sandbox ..."
    mv $creamSandboxPath ${creamSandboxPath}_old
    rm -rf ${creamSandboxPath}_old &
  fi
else
  echo "The $3 database is synchronized with the one requested by cream service"
fi

