/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream.client;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.rpc.ServiceException;

import org.glite.security.delegation.Delegation;
import org.glite.security.delegation.DelegationException;

public class JobProxyRenew extends JobCommand {

    public JobProxyRenew(String[] args, List<String> options) throws RuntimeException {
        super(args, options);
    }

    protected void execute() {
        if(getDelegationId() == null) {
            printUsage();
        }
        
        try {
            Delegation delegation = getCreamDelegation();
            String pkcs10 = delegation.renewProxyReq(getDelegationId());
            String request = signRequest(pkcs10, getDelegationId());
            delegation.putProxy(getDelegationId(), request);
        } catch (ServiceException e) {
            System.out.println("ServiceException: " + e.getMessage());
        } catch (DelegationException e) {
            System.out.println("DelegationException: " + e.getMsg());
        } catch (RemoteException e) {
            System.out.println("RemoteException: " + e.getMessage());
        } catch (IOException e) {
            System.out.println("IOException: " + e.getMessage());
        }        
    }

    public static void main(String[] args) {
        List<String> options = new ArrayList<String>();
        options.add(EPR);
        options.add(DELEGATION_ID);
        
        new JobProxyRenew(args, options);
    }
}
