/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.db;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.glite.ce.creamapi.cmdmanagement.Command;

/**
 * Class providing utilities about commands.
 *
 */
public class CommandHelper {
    
    /** The logger */
    private static Logger logger = Logger.getLogger(CommandHelper.class);
    
    /**
     * Returns the list of command parameters.
     * @param cmd The command.
     * @return The list of command parameters.
     */
    public static List<CommandParameter> getParameterCommands(Command cmd) {
        ArrayList<CommandParameter> parameters = new ArrayList<CommandParameter>();
        if (cmd != null) {
            Set keys = cmd.getParameterKeySet();
            if (keys != null) {
                Iterator i = keys.iterator();
                while (i.hasNext()) {
                    String name = (String) i.next();
                    List<String> valList = cmd.getParameterMultivalue(name);
                    if (valList != null) {
                        Iterator<String> it = valList.iterator();
                        while (it.hasNext()) {
                            String val = it.next();
                            parameters.add(new CommandParameter(name, val));
                        }
                    }                    
                }
            }
        }
        return parameters;
    }
    
    /**
     * Set the command parameters in the command.
     * @param cmd The command to which parameters are added.
     * @param parameters The list of command parameters.
     */
    public static void setParameters(Command cmd, List<CommandParameter> parameters) {
        if (cmd != null) {
            if (parameters != null) {
                // Create multivalue map for the command parameters
                Map<String, List<String>> map = new HashMap<String, List<String>>();
                Iterator<CommandParameter> i = parameters.iterator();
                CommandParameter parameter = null;
                while (i.hasNext()) {
                    parameter = i.next();
                    List<String> l = map.get(parameter.getName());
                    if (l == null) {
                        map.put(parameter.getName(), l = new ArrayList<String>());
                    }
                    l.add(parameter.getValue());
                }
                
                // Iterate over the map to set command paramters
                Iterator<String> keys = map.keySet().iterator();
                while (keys.hasNext()) {
                    String key  = keys.next();
                    List<String> values = map.get(key);
                    if (values != null) {
                        if (values.size() == 1) {
                            cmd.addParameter(key, values.get(0));
                        } else {
                            cmd.addParameter(key, values);
                        }
                    }
                }
            }
            
        }
    }
}
