/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.db;

import java.io.Serializable;

/**
 * Class representing a command parameter. 
 *
 */
public class CommandParameter implements Serializable {
    private static final long serialVersionUID = 1L;
    
    /** id identifying a command parameter */
    private long id = -1;
    
    /** The command id associated to a command parameter */
    private long commandId = -1;
    
    /** The name of the command parameter */
    private String name = null;
    
    /** The value of the command parameter */
    private String value = null;

    /**
     * Constructor.
     * @param name The command parameter name.
     * @param value The command parameter value.
     */
    public CommandParameter(String name, String value) {
        this.name = name;
        this.value = value;
    }

    /**
     * @return the id
     */
    public long getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(long id) {
        this.id = id;
    }

    /**
     * @return the commandId
     */
    public long getCommandId() {
        return commandId;
    }

    /**
     * @param commandId the commandId to set
     */
    public void setCommandId(long commandId) {
        this.commandId = commandId;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the value
     */
    public String getValue() {
        return value;
    }

    /**
     * @param value the value to set
     */
    public void setValue(String value) {
        this.value = value;
    }
}
