/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.glite.ce.common.db.TableInterface;

/**
 * Interface to access the command_parameter table in the database.
 *
 */
public interface CommandParameterTableInterface extends TableInterface {
    
    /**
     * Inserts a given command parameter into the db.
     * If a successful insert occurs the id (a generated key) is saved
     * in the command parameter as a side effect result.
     * @param cmd The command to be inserted.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeInsert(CommandParameter parameter, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Deletes a command parameter identified by <code>parameterId</code> from the db.
     * @param parameterId The id of the command parameter to be deleted.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeDelete(long parameterId, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Retrieves the command parameter info from the db.
     * @return The command parameter associated to a given commandId. 
     * @param parameterId The id of the command parameter to be selected.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public CommandParameter executeSelect(long parameterId, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Returns the list of command parameters associated to a given commandId.
     * @return The list of command parameters associated to a given commandId.
     * @param commandId The id of the command.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public List<CommandParameter> findByCommandId(long commandId, Connection connection) throws SQLException, IllegalArgumentException;
}
