/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;

import org.glite.ce.common.db.TableInterface;
import org.glite.ce.creamapi.cmdmanagement.Command;

/**
 * Interface to access the command table in the database.
 *
 */
public interface CommandTableInterface extends TableInterface {
    
    /**
     * Inserts a given command into the db.
     * If a successful insert occurs the id (a generated key) is saved
     * in the parameter Command as a side effect result.
     * @param cmd The command to be inserted.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeInsert(Command cmd, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Deletes a command identified by <code>commandId</code> from the db.
     * @param commandId The id of the command to be deleted.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeDelete(String commandId, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Updates a command into the db.
     * @param cmd The command to be updated.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeUpdate(Command cmd, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Retrieves the command info from the db.
     * @param commandId The id of the command to be selected.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public Command executeSelect(String commandId, Connection connection) throws SQLException, IllegalArgumentException;
}
