/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.queue.db;

import java.io.Serializable;

import org.glite.ce.creamapi.cmdmanagement.Command;

/**
 * Class representing an item in a command queue. 
 *
 */
public class CommandQueueItem implements Serializable {
    private static final long serialVersionUID = 1L;
    
    public static final boolean IN_EXECUTION = true;
    
    public static final boolean NOT_SCHEDULED = false;
    
    /** id identifying an item in a command queue */
    private long id = -1;
    
    /** the command id related to an item in a command queue */
    private long commandId = -1;
   
    /** the scheduling status */
    private boolean isScheduled = NOT_SCHEDULED;
    
    /** priority level in a command queue */
    private int priorityLevel = 0;
    
    /** commandGroupId in a command queue */
    private String commandGroupId = null;
    
    /** executionMode of the command. Default is SERIAL */
    private Command.ExecutionModeValues executionMode = Command.ExecutionModeValues.SERIAL;

    /**
     * @return the id
     */
    public long getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(long id) {
        this.id = id;
    }

    /**
     * @return the commandId
     */
    public long getCommandId() {
        return commandId;
    }

    /**
     * @param commandId the commandId to set
     */
    public void setCommandId(long commandId) {
        this.commandId = commandId;
    }

    /**
     * @return the isScheduled
     */
    public boolean isScheduled() {
        return isScheduled;
    }

    /**
     * @param isScheduled the isScheduled to set
     */
    public void setScheduled(boolean isScheduled) {
        this.isScheduled = isScheduled;
    }

    /**
     * @return the priorityLevel
     */
    public int getPriorityLevel() {
        return priorityLevel;
    }

    /**
     * @param priorityLevel the priorityLevel to set
     */
    public void setPriorityLevel(int priorityLevel) {
        this.priorityLevel = priorityLevel;
    }

	/**
	 * @return commandGroupId commandGroupId of the command
	 */
	public String getCommandGroupId() {
		return commandGroupId;
	}

	/**
	 * @param commandGroupId commandGroupId of the command
	 */
	public void setCommandGroupId(String commandGroupId) {
		this.commandGroupId = commandGroupId;
	}

	/**
	 * @return the execution mode of the command
	 */
	public Command.ExecutionModeValues getExecutionMode() {
		return executionMode;
	}

	/**
	 * @param executionMode the execution mode of the command
	 */
	public void setExecutionMode(Command.ExecutionModeValues executionMode) {
		this.executionMode = executionMode;
	} 
}
