/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 *
 */

package org.glite.ce.cream.cmdmanagement.queue.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.glite.ce.common.db.TableInterface;

/**
 * Interface to access the command table in the database.
 *
 */
public interface CommandQueueListTableInterface extends TableInterface {
    
    /**
     * Inserts a queue name in the list of queues.
     * @param queueName The queue name to be inserted.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeInsert(String queueName, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Deletes a queue name from the list of queues.
     * @param queueName The queue name to be inserted.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeDelete(String queueName, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Checks if a queue name exists in the list of queues.
     * @connection The connection.
     * @return Returns true if the queue exists, false otherwise.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public boolean executeIsQueueNamePresent(String queueName, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Returns the list of queue names registered at a given time.
     * If no matching occurs the returned list is empty. 
     * @connection The connection.
     * @return Returns the list of queue names registered at a given time.
     * If no matching occurs the returned list is empty.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public List<String> executeFindQueueNames(Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Executes SQL code to create the table for a given queue name.
     * @param queueName The table to be created for a given queue name.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeTableCreation(String queueName, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Executes SQL code to drop the table for a given queue name.
     * @param queueName The table to be dropped for a given queue name.
     * @connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeTableDrop(String queueName, Connection connection) throws SQLException, IllegalArgumentException;
}
