/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
package org.glite.ce.cream.configuration;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.glite.ce.commonj.configuration.CEConfigResource;

public class CommandExecutorConfig implements CEConfigResource {
    private static final long serialVersionUID = 1L;
    protected String name;
    protected String category;
    protected String jarFileName;
    protected Hashtable<String, Property> properties;

    public CommandExecutorConfig() {
        properties = new Hashtable<String, Property>(0);
    }

    public void addProperty(Property property) throws IllegalArgumentException {
        if (property == null) {
            return;
        }
        if (properties.containsKey(property.getKey())) {
            throw new IllegalArgumentException("the property \"" + property.getKey() + "\" already exists!");
        }

        properties.put(property.getKey(), property);
    }

    public Object clone() {
        CommandExecutorConfig result = new CommandExecutorConfig();
        result.setName(name);
        result.setCategory(category);
        result.setJarFileName(jarFileName);

        for (Property property : properties.values()) {
            result.addProperty(new Property(property.getKey(), property.getValue()));
        }

        return result;
    }

    public String getCategory() {
        return category;
    }

    public String getJarFileName() {
        return jarFileName;
    }

    public String getName() {
        return name;
    }

    public List<Property> getProperties() {
        return new ArrayList<Property>(properties.values());
    }

    public Property getProperty(String name) {
        return properties.get(name);
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public void setJarFileName(String jarFileName) {
        this.jarFileName = jarFileName;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setProperties(List<Property> propertyList) {
        properties.clear();

        for (Property property : propertyList) {
            addProperty(property);
        }
    }
}
