/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
package org.glite.ce.cream.configuration;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class ConfigurationParser extends DefaultHandler {
    private ServiceConfig service;    
    private String tmpVal;
    private Property tmpProperty;
    private CommandExecutorConfig cmdExec;
    
    /**
     * Iterate through the list and print the contents
     */
    private void printData() {
        List<Property> properties = service.getProperties();
        for(int i=0; i<properties.size(); i++) {
            System.out.println("service property: key=" + properties.get(i).getKey() + " value=" + properties.get(i).getValue());            
        }
        
        List<CommandExecutorConfig> cmdExecutor = service.getCommandExecutor();
        for(int i=0; i<cmdExecutor.size(); i++) {
            System.out.println("cmdExecutor: name=" + cmdExecutor.get(i).getName() + " category=" + cmdExecutor.get(i).getCategory()); 

            properties = cmdExecutor.get(i).getProperties();
            for(int x=0; x<properties.size(); x++) {
                System.out.println("    property: key=" + properties.get(x).getKey() + " value=" + properties.get(x).getValue());            
            }
        }
        
    }

    // Event Handlers
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        // reset
        tmpVal = "";
        if (qName.equalsIgnoreCase("Service")) { 
            service = new ServiceConfig();
        } else if (qName.equalsIgnoreCase("Properties")) { 
           
        } else if (qName.equalsIgnoreCase("Entry")) {
            if(tmpProperty != null) {
                throw new SAXException("Entry tag not closed!");                
            }
            tmpProperty = new Property();
           // tmpEmp.setType(attributes.getValue("type"));
        } else if (qName.equalsIgnoreCase("key")) {
            
        } else if (qName.equalsIgnoreCase("value")) {
            
        } else if (qName.equalsIgnoreCase("CommandExecutors")) {
            
        } else if (qName.equalsIgnoreCase("CommandExecutor")) {
            if(cmdExec != null) {
                throw new SAXException("CommandExecutor tag not closed!");                
            }
            cmdExec = new CommandExecutorConfig();
        }
    }

    public void characters(char[] ch, int start, int length) throws SAXException {
        tmpVal = new String(ch, start, length);
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (qName.equalsIgnoreCase("Properties")) {
          //  serviceProperty = false;
        } else if (qName.equalsIgnoreCase("Entry")) {
            if(tmpProperty.getKey() == null) {
                throw new SAXException("Entry key not defined!");
            }
            if(tmpProperty.getValue() == null) {
                throw new SAXException("Entry value not defined!");
            }
            if(cmdExec == null) {
                service.addProperty(tmpProperty);
            } else {
                cmdExec.addProperty(tmpProperty);
            }
            tmpProperty = null;
        } else if (qName.equalsIgnoreCase("key")) {
            tmpProperty.setKey(tmpVal);
        } else if (qName.equalsIgnoreCase("value")) {
            tmpProperty.setValue(tmpVal);
        } else if (qName.equalsIgnoreCase("name")) {
            cmdExec.setName(tmpVal);
        } else if (qName.equalsIgnoreCase("category")) {
            cmdExec.setCategory(tmpVal);
        } else if (qName.equalsIgnoreCase("jarFileName")) {
            cmdExec.setJarFileName(tmpVal);
        } else if (qName.equalsIgnoreCase("CommandExecutor")) {
            if(cmdExec.getJarFileName() == null) {
                throw new SAXException("CommandExecutorConfig jarFileName not defined!");
            }
            service.addCommandExecutor(cmdExec);
            cmdExec = null;
        }
    }

    public ServiceConfig parseDocument(String file) throws IllegalArgumentException, RuntimeException {
        if(file == null) {
            throw new IllegalArgumentException("configuration file not specified!");
        }
        
        // get a factory
        SAXParserFactory spf = SAXParserFactory.newInstance();
        try {
            // get a new instance of parser
            SAXParser sp = spf.newSAXParser();

            // parse the file and also register this class for call backs
            sp.parse(file, this);
        } catch (SAXException se) {
            throw new RuntimeException("Configuration parser error: " + se.getMessage());
        } catch (ParserConfigurationException pce) {
            throw new RuntimeException("Configuration parser error: " + pce.getMessage());
        } catch (IOException ie) {
            throw new RuntimeException("Configuration parser error: " + ie.getMessage());
        }
        
        return service;
    }
    
    
    public static void main(String[] args){
        ConfigurationParser cp = new ConfigurationParser();
        cp.parseDocument("/home/zangrand/workspace/cream/cream-config.xml");
        cp.printData();
    }
}
