/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.cream.delegationmanagement;

import java.util.List;
import java.util.TimerTask;

import org.apache.log4j.Logger;
import org.glite.ce.cream.cmdmanagement.CommandManager;
import org.glite.ce.cream.jobmanagement.command.DeleteProxyFromSandboxCmd;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.CommandManagementException;
import org.glite.ce.creamapi.delegationmanagement.Delegation;
import org.glite.ce.creamapi.delegationmanagement.DelegationException;
import org.glite.ce.creamapi.delegationmanagement.DelegationManagementException;

public class DelegationPurger extends TimerTask {
    private final static Logger logger = Logger.getLogger(DelegationPurger.class.getName());

    public void run() {
        logger.info("Begin DelegationPurger");
        try {
            List<Delegation> expiredDelegationList = DelegationManager.getInstance().getExpiredDelegation();

            for (Delegation delegation : expiredDelegationList) {
                logger.debug("deleting expired delegation [delegationId=" + delegation.getId() + "] [DN=" + delegation.getDN() + "] [localUser=" + delegation.getLocalUser() + "] [ExpirationTime=" + delegation.getExpirationTime() + "]");

                DelegationManager.getInstance().deleteDelegation(delegation.getId(), delegation.getDN(), delegation.getLocalUser());                

                try {
                    DeleteProxyFromSandboxCmd cmd = new DeleteProxyFromSandboxCmd();
                    cmd.setUserId(delegation.getUserId());
                    cmd.setUserDN(delegation.getDN());
                    cmd.setUserFQAN(delegation.getFQAN());
                    cmd.setLocalUser(delegation.getLocalUser());
                    cmd.setLocalUserGroup(delegation.getLocalUserGroup());
                    cmd.setDelegationProxyId(delegation.getId());
                    //cmd.setDelegationProxySuffix(delegationSuffix);
                    cmd.setCommandGroupId(delegation.getUserId());

                    CommandManager.getInstance().insertCommand(cmd);
                } catch (CommandException e) {
                    logger.error("DelegationPurger CommandException: " + e.getMessage());
                    return;
                } catch (CommandManagementException e) {
                    logger.error("DelegationPurger CommandManagementException: " + e.getMessage());
                    return;
                }
                                
                logger.info("deleted expired delegation [delegationId=" + delegation.getId() + "] [DN=" + delegation.getDN() + "] [localUser=" + delegation.getLocalUser() + "] [ExpirationTime=" + delegation.getExpirationTime() + "]");
            }
        } catch (DelegationException e) {
            logger.error(e.getMessage());
        } catch (IllegalArgumentException e) {
            logger.error(e.getMessage());
        } catch (DelegationManagementException e) {
            logger.error(e.getMessage());
        } finally {
            logger.info("End DelegationPurger");
        }
    }
}
