/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando, <luigi.zangrando@pd.infn.it>
 *
 */

package org.glite.ce.cream.jobmanagement.cmdexecutor.dummy;

import java.util.Calendar;

import org.apache.log4j.Logger;
import org.glite.ce.cream.jobmanagement.db.JobDBInMemory;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.CommandResult;
import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.cmdexecutor.AbstractJobExecutor;


public final class DummyExecutor extends AbstractJobExecutor {
    private static Logger logger = Logger.getLogger(DummyExecutor.class.getName());

    public DummyExecutor() {
        super("DummyExecutor");

        setJobDB(new JobDBInMemory());
        
        //setJobDB(new JobDBImplementation());
    }

    public void destroy() {
    }

    public void initExecutor() throws CommandException {
     //   getJobDB().init(null);   
    }

    public boolean isLRMSSupported(String arg0) {
        return arg0.equalsIgnoreCase("dummy");
    }

    public CommandResult getStatus(Job arg0) {
        return new CommandResult();
    }

    public void cancel(Job job) throws CommandException {
        try {
            Thread.sleep(10000);
            doOnJobStatusChanged(new JobStatus(JobStatus.CANCELLED, job.getId()));
        } catch(Exception e) {
            logger.error(e);
        }
      // throw new CommandException("ahhhhhh");
    }

    public void renewProxy(Job job) throws CommandException {
    }

    public void renewProxy(Job job, boolean sendToWN) throws CommandException {
    }

    public void resume(Job job) throws CommandException {
        try {
            Thread.sleep(30000);
            doOnJobStatusChanged(new JobStatus(job.getStatusAt(job.getStatusCount()-2).getType(), job.getId()));
        } catch(Exception e) {
            logger.error(e);
        }
    }

    public CommandResult submit(Job job) throws CommandException {
        CommandResult result = new CommandResult();
        
        if (job == null) {
            throw new CommandException("job not defined!");
        }

        Calendar now = Calendar.getInstance();
        result.addParameter("LRMS_JOB_ID", "" + now.getTime());
        result.addParameter("LRMS_ABS_JOB_ID",  "dummyId_"+ now.getTime());
        
        return result;
    }

    public void suspend(Job job) throws CommandException {
        try {
            Thread.sleep(30000);
            doOnJobStatusChanged(new JobStatus(JobStatus.HELD, job.getId()));
        } catch(Exception e) {
            logger.error(e);
        }
    }

    @Override
    public boolean isBatchSystemSupported(String bs) {
        return true;
    }

}
