/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando, <luigi.zangrando@pd.infn.it>
 *
 */

package org.glite.ce.cream.ws;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;
import java.nio.channels.OverlappingFileLockException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TimeZone;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectInstance;
import javax.management.ObjectName;

import org.apache.axis.MessageContext;
import org.apache.axis.message.SOAPHeaderElement;
import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;
import org.glite.ce.common.utils.JarClassLoader;
import org.glite.ce.commonj.utils.CEUtils;
import org.glite.ce.cream.cmdmanagement.CommandManager;
import org.glite.ce.cream.configuration.CommandExecutorConfig;
import org.glite.ce.cream.configuration.Property;
import org.glite.ce.cream.configuration.ServiceConfig;
import org.glite.ce.cream.configuration.CREAMConfigConstants.ServiceAttribute;
import org.glite.ce.cream.dbmanagement.DBInfoManager;
import org.glite.ce.cream.delegationmanagement.DelegationManager;
import org.glite.ce.cream.jobmanagement.command.CopyNewProxyToSandboxCmd;
import org.glite.ce.cream.jobmanagement.command.DeleteLeaseCmd;
import org.glite.ce.cream.jobmanagement.command.DeleteProxyFromSandboxCmd;
import org.glite.ce.cream.jobmanagement.command.GetLeaseCmd;
import org.glite.ce.cream.jobmanagement.command.GetServiceInfo;
import org.glite.ce.cream.jobmanagement.command.JobCancelCmd;
import org.glite.ce.cream.jobmanagement.command.JobCmd;
import org.glite.ce.cream.jobmanagement.command.JobInfoCmd;
import org.glite.ce.cream.jobmanagement.command.JobListCmd;
import org.glite.ce.cream.jobmanagement.command.JobPurgeCmd;
import org.glite.ce.cream.jobmanagement.command.JobRegisterCmd;
import org.glite.ce.cream.jobmanagement.command.JobResumeCmd;
import org.glite.ce.cream.jobmanagement.command.JobSetLeaseIdCmd;
import org.glite.ce.cream.jobmanagement.command.JobStartCmd;
import org.glite.ce.cream.jobmanagement.command.JobStatusCmd;
import org.glite.ce.cream.jobmanagement.command.JobSuspendCmd;
import org.glite.ce.cream.jobmanagement.command.QueryEventCmd;
import org.glite.ce.cream.jobmanagement.command.SetAcceptNewJobsCmd;
import org.glite.ce.cream.jobmanagement.command.SetLeaseCmd;
import org.glite.ce.cream.jobmanagement.command.UpdateProxyToSandboxCmd;
import org.glite.ce.cream.ws.delegation.CREAMDelegationService;
import org.glite.ce.cream.ws.delegation.DelegationProxyEvent;
import org.glite.ce.cream.ws.delegation.DelegationProxyInfo;
import org.glite.ce.cream.ws.delegation.DelegationProxyListener;
import org.glite.ce.cream.ws.delegation.storage.CREAMGrDPStorageInterface;
import org.glite.ce.cream.ws.utils.FaultFactory;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.CommandExecutor;
import org.glite.ce.creamapi.cmdmanagement.CommandManagementException;
import org.glite.ce.creamapi.cmdmanagement.Command.ExecutionModeValues;
import org.glite.ce.creamapi.delegationmanagement.Delegation;
import org.glite.ce.creamapi.delegationmanagement.DelegationManagementException;
import org.glite.ce.creamapi.eventmanagement.Event;
import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.JobEnumeration;
import org.glite.ce.creamapi.jobmanagement.JobManagementException;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.cmdexecutor.JobCommandConstant;
import org.glite.ce.creamapi.jobmanagement.cmdexecutor.JobSubmissionManager;
import org.glite.ce.creamapi.jobmanagement.cmdexecutor.JobSubmissionManagerInfo;
import org.glite.ce.creamapi.jobmanagement.command.JobCommand;
import org.glite.ce.creamapi.jobmanagement.command.JobIdFilterFailure;
import org.glite.ce.creamapi.jobmanagement.command.JobIdFilterResult;
import org.glite.ce.creamapi.ws.cream2.CREAMPort;
import org.glite.ce.creamapi.ws.cream2.types.AuthorizationFault;
import org.glite.ce.creamapi.ws.cream2.types.Command;
import org.glite.ce.creamapi.ws.cream2.types.DateMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.DelegationIdMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.GenericFault;
import org.glite.ce.creamapi.ws.cream2.types.InvalidArgumentFault;
import org.glite.ce.creamapi.ws.cream2.types.JobDescription;
import org.glite.ce.creamapi.ws.cream2.types.JobFilter;
import org.glite.ce.creamapi.ws.cream2.types.JobId;
import org.glite.ce.creamapi.ws.cream2.types.JobInfo;
import org.glite.ce.creamapi.ws.cream2.types.JobInfoResult;
import org.glite.ce.creamapi.ws.cream2.types.JobRegisterResult;
import org.glite.ce.creamapi.ws.cream2.types.JobSetLeaseIdRequest;
import org.glite.ce.creamapi.ws.cream2.types.JobStatusInvalidFault;
import org.glite.ce.creamapi.ws.cream2.types.JobStatusResult;
import org.glite.ce.creamapi.ws.cream2.types.JobSubmissionDisabledFault;
import org.glite.ce.creamapi.ws.cream2.types.JobUnknownFault;
import org.glite.ce.creamapi.ws.cream2.types.Lease;
import org.glite.ce.creamapi.ws.cream2.types.LeaseIdMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.OperationNotSupportedFault;
import org.glite.ce.creamapi.ws.cream2.types.QueryEventRequest;
import org.glite.ce.creamapi.ws.cream2.types.QueryEventResult;
import org.glite.ce.creamapi.ws.cream2.types.Result;
import org.glite.ce.creamapi.ws.cream2.types.ServiceInfo;
import org.glite.ce.creamapi.ws.cream2.types.ServiceMessage;
import org.glite.ce.creamapi.ws.cream2.types.Status;
import org.glite.ce.faults.AuthenticationFault;
import org.glite.security.delegation.DelegationException;

public class CREAM2Service implements CREAMPort, DelegationProxyListener {

    private static long dbID;
	
    private static final Logger logger = Logger.getLogger(CREAM2Service.class.getName());

    protected static boolean initialized = false;

    protected static CREAMDelegationService delegationService;

    protected static URI creamURL;
    
    protected static ServiceInfo serviceInfo = null;

    protected static String proxyDir = null;
    
    protected static String delegationSuffix = null;


    /**
     * Enables/Disables the CREAM service for accepting new job submissions. Only the
     * CREAM administrator can invoke this operation.
     * 
     * @throws RemoteException
     * @throws AuthenticationFault
     *             thrown in case of authentication problems.
     * @throws AuthorizationFault
     *             thrown in case of authorization problems.
     * @throws GenericFault
     *             thrown if any other possible error occurs.
     */
    public void acceptNewJobSubmissions(boolean b) throws RemoteException, GenericFault, AuthorizationFault {
        if (!CEUtils.isAdmin()) {
            throw FaultFactory.makeGenericFault("acceptNewJobSubmissions", "0", "Operation reserved only to administrator!", "Operation reserved only to administrator!");
        }

        SetAcceptNewJobsCmd cmd = new SetAcceptNewJobsCmd(b);
        cmd.setUserId(getUserDN_FQAN(null));
        cmd.setIsAdmin(true);

        try {
            CommandManager.getInstance().insertCommand(cmd);
            serviceInfo.setDoesAcceptNewJobSubmissions(b);
        } catch (Throwable e) {       
            throw FaultFactory.makeGenericFault("acceptNewJobSubmissions", "0", e.getMessage(), "N/A");
        }        
    }


    protected void initWorkingDir(String cream_scratch_dir) throws IOException {
        CEUtils.makeDir(cream_scratch_dir + File.separator + "user_proxy");

        Runtime runtime = Runtime.getRuntime();
        Process proc = runtime.exec("chmod 700 " + cream_scratch_dir);
        
        try {
            proc.waitFor();
        } catch (InterruptedException e) {
        } finally {
            try {
                proc.getInputStream().close();
            } catch (IOException e) {}
            try {
                proc.getErrorStream().close();
            } catch (IOException e) {}
            try {
                proc.getOutputStream().close();
            } catch (IOException e) {}
        }
    }
 
    private CREAMDelegationService getDelegationService() throws InstantiationException {
        logger.debug("Begin getDelegationService");
        if (delegationService == null) {
            try {
                logger.info("configuring the delegationService");

                delegationService = new CREAMDelegationService();
                
                if (delegationService.getGrDPStorage() instanceof CREAMGrDPStorageInterface) {
                    ((CREAMGrDPStorageInterface)delegationService.getGrDPStorage()).addDelegationProxyListener(this);
                }

                logger.info("configuring the delegationService: done");
            } catch (Throwable e) {
                logger.error("DelegationService configuration error: " + e.getMessage());
                throw new InstantiationException("DelegationService configuration error: " + e.getMessage());
            }
        }
        logger.debug("End getDelegationService");
        return delegationService;
    }
  
//    private DelegationProxyInfo getProxy(String delegationId, String userDN, String localUser) throws Exception {
//        DelegationProxyInfo delegProxyInfo = (DelegationProxyInfo) MessageContext.getCurrentContext().getProperty(Constants.PROXYCERT_INFO);
//
//        if (delegProxyInfo == null) {
//            try {
//                delegProxyInfo = ((CREAMGrDPStorageInterface)getDelegationService().getGrDPStorage()).getDelegationProxy(delegationId, userDN, localUser);
//            } catch (Exception e) {
//                e.printStackTrace();
//                throw new RuntimeException(e.getMessage());
//            }
//        }
//
//        if (delegProxyInfo == null) {
//            throw (new Exception("delegProxyInfo \"" + delegationId + "\" not found!"));
//        }
//
//        return delegProxyInfo;
//    }

    protected String getUserDN_FQAN(String vo) throws GenericFault {
        String fqan = "";
        List<String> fqanlist = null;

        if (vo != null) {
            fqanlist = CEUtils.getFQAN(vo);
        } else {
            String userVO = CEUtils.getUserDefaultVO();

            if (userVO != null) {
                fqanlist = CEUtils.getFQAN(userVO);
            }
        }

        if (fqanlist != null && fqanlist.size() > 0) {
            fqan = normalize(fqanlist.get(0).toString());
        }

        return normalize(CEUtils.getUserDN_X500()) + fqan;
    }

    private String getHTTPTomcatPort() {
        logger.debug("Begin getHTTPTomcatPorts");
       // StringBuffer httpTomcatPorts = new StringBuffer();
        String serverPort = null;

        try {
            //Get MBean server reference
            ArrayList servers = MBeanServerFactory.findMBeanServer(null);
            if (servers == null) {
                throw new Exception("No MBeanServer found.");
            }

            MBeanServer server = (MBeanServer)servers.get(0);
            Set mbeanSet = server.queryMBeans(null, null);

            ObjectName objnamestr = null;
            Iterator i = mbeanSet.iterator();
            
            while (i.hasNext()) {
                objnamestr = ((ObjectInstance) i.next()).getObjectName();
                if (("Catalina".equals(objnamestr.getDomain())) && (objnamestr.getCanonicalName().indexOf("type=Connector")!= -1)) {
                    logger.debug("Found Tomcat connector");
                    String protocol = (String)server.getAttribute(objnamestr, "protocol");

                    if (protocol.toUpperCase().indexOf("HTTP") != -1) {
                        serverPort = ((Integer)server.getAttribute(objnamestr, "port")).toString();
                        logger.info("Found HTTP Tomcat port = " + serverPort);
            //            httpTomcatPorts = httpTomcatPorts.append(serverPort);
                        break;
                    }
                }
            } //end while
        } catch (Exception e) {
            logger.warn("It's not possible to find http tomcat port: "  + e.getMessage());
            logger.warn("The default Tomcat port (8443) will be used.");
            serverPort = "8443";
         //   httpTomcatPorts = new StringBuffer("8443");
        }

        logger.debug("End getHTTPTomcatPorts");

        return serverPort;  
    }

    private void checkDBVersion(String dbName, String datasource, String databaseVersion)throws InstantiationException{
    	String databaseVersionFromDatabase = DBInfoManager.getDBVersion(datasource);
    	if (databaseVersionFromDatabase == null) {
    		String errorMsg = "Error: cannot retrieve the " + dbName + " database version from the database " +
    		"(Requested version is " + databaseVersion + ") because either the database isn't reachable " +
    		"or the database version isn't correct.";
    		logger.error(errorMsg);
    		throw new InstantiationException(errorMsg);
    	} else {
    		if (!databaseVersionFromDatabase.equals(databaseVersion)) {
    			String errorMsg = "The " + dbName + " database version (db version = " + 
    			databaseVersionFromDatabase + ") is not compliant with the one requested by cream service (db version = " + 
    			databaseVersion + ")."; 
    			logger.error(errorMsg);
    			throw new InstantiationException(errorMsg);
    		} else {
    			logger.info(dbName + " database version is correct.");
    		}
    	}
    }

    public void init() throws InstantiationException, InvocationTargetException {
        if (!initialized) {
            logger.info("Initializing the CREAMService...");

            if (serviceInfo == null) {
                serviceInfo = new ServiceInfo();
            }

            serviceInfo.setStartupTime(Calendar.getInstance());
            serviceInfo.setDoesAcceptNewJobSubmissions(false);
            serviceInfo.setStatus("INIT");

            try {
                creamURL = new URI("https://" + InetAddress.getLocalHost().getCanonicalHostName() + ":" + getHTTPTomcatPort() + "/ce-cream/services/CREAM2");
                logger.info("CREAM URL: " + creamURL);
            } catch (UnknownHostException e) {
                throw new InstantiationException("Configuration error (creamURL): " + e.getMessage());
            } catch (MalformedURIException e) {
                throw new InstantiationException("Configuration error (creamURL): " + e.getMessage());
            }

            ServiceConfig service = ServiceConfig.getConfiguration();
            if (service==null) {
            	throw new InstantiationException("Cannot initialize service configuration");
            }

            String description = service.getStringParameter(ServiceAttribute.CREAM_DESCRIPTION_LABEL);
            if (description != null) {
                serviceInfo.setDescription(description);
            } else {
                serviceInfo.setDescription("CREAM2");                
            }
            
            String interfaceVersion = service.getStringParameter(ServiceAttribute.CREAM_INTERFACE_VERSION_LABEL);
            if (interfaceVersion != null) {
                serviceInfo.setInterfaceVersion(interfaceVersion);
            } else {
                serviceInfo.setInterfaceVersion("N/A");                
            }

            String serviceVersion = service.getStringParameter(ServiceAttribute.CREAM_SERVICE_VERSION_LABEL);
            if (serviceVersion != null) {
                serviceInfo.setServiceVersion(serviceVersion);
            } else {
                serviceInfo.setServiceVersion("N/A");                
            }

            //check creamdb version
            String creamdbVersion = service.getStringParameter(ServiceAttribute.CREAMDB_DATABASE_VERSION_LABEL);
            if (creamdbVersion == null) {
                logger.error("Configuration error: Cannot retrieve the creamdb_database_version property from configuration");
                throw new InstantiationException("Configuration error: Cannot retrieve the creamdb_database_version property from configuration");
            } else {
                checkDBVersion("creamdb", "java:comp/env/jdbc/creamdb", creamdbVersion); 
            } 
            
            //check delegationdb version
            String delegationdbVersion = service.getStringParameter(ServiceAttribute.DELEGATIONDB_DATABASE_VERSION_LABEL);
            if (delegationdbVersion == null) {
                logger.error("Configuration error: Cannot retrieve the delegationdb_database_version property from configuration");
                throw new InstantiationException("Configuration error: Cannot retrieve the delegationdb_database_version property from configuration");
            } else {
                checkDBVersion("delegationdb", "java:comp/env/jdbc/dlg", delegationdbVersion);
            }
            
            //retrieve DBID, that is the creationTime of the cream database.
            dbID = DBInfoManager.getCreationTime("java:comp/env/jdbc/creamdb");
            logger.info("DB identifier = " + dbID);
            if (dbID == -1) {
                String errorMsg = "Error: cannot retrieve the the DB identifier from database";
                logger.error(errorMsg);
                throw new InstantiationException(errorMsg);
            }

            logger.info("init CommandManager");
            try {
                CommandManager.getInstance();
            } catch (CommandManagementException e) {
                throw new InstantiationException("Configuration error: " + e.getMessage());
            }
            logger.info("init CommandManager: done");

            List<CommandExecutorConfig> cmdExecutorList = service.getCommandExecutorList();
            if (cmdExecutorList == null || cmdExecutorList.size() == 0) {
                throw new InstantiationException("Configuration error: none CommandExecutor defined!");
            }

            try {
                CommandExecutorConfig cmdExecutorConfig = null;
                for (int i = 0; i < cmdExecutorList.size(); i++) {
                    cmdExecutorConfig = cmdExecutorList.get(i);
                    String jarFileName = cmdExecutorConfig.getJarFileName();

                    logger.info("initializing a new CommandExecutor: name = " + cmdExecutorConfig.getName() + "; category = " + cmdExecutorConfig.getCategory()
                            + "; JarFileName = " + cmdExecutorConfig.getJarFileName());

                    Class c = makeCommandExecutorInstance(jarFileName);

                    if (c == null) {
                        throw new RuntimeException("Cannot create the CommandExecutor class from path " + jarFileName);
                    }

                    Object obj = c.newInstance();

                    if (obj instanceof CommandExecutor) {
                        CommandExecutor cmdExecutor = (CommandExecutor) obj;

                        logger.info("New CommandExecutor found!: name = " + cmdExecutor.getName() + "; category = " + cmdExecutor.getCategory());

                        List<Property> properties = cmdExecutorConfig.getProperties();
                        for (int x = 0; x < properties.size(); x++) {
                            cmdExecutor.addParameter(properties.get(x).getKey(), properties.get(x).getValue());
                            logger.info("CommandExecutor property: " + properties.get(x).getKey() + " = \"" + properties.get(x).getValue() + "\"");
                        }

                        CommandManager.getInstance().addCommandExecutor(cmdExecutor);
                    }
                }
            } catch (IllegalArgumentException e) {
                throw new InstantiationException("error: " + e.getMessage());
            } catch (CommandManagementException e) {
                throw new InstantiationException("CommandManagement error: " + e.getMessage());
            } catch (IllegalAccessException e) {
                throw new InstantiationException("error: " + e.getMessage());
            }
            
            if ((proxyDir = service.getStringParameter(ServiceAttribute.JOB_DIR_LABEL)) == null) {
                throw new InstantiationException("Configuration error: " +  ServiceAttribute.JOB_DIR_LABEL + " property not defined!");
            }

            proxyDir += File.separator + "user_proxy";
            
            try {
            	initWorkingDir(service.getStringParameter(ServiceAttribute.JOB_DIR_LABEL));
            } catch (IOException e1) {
                throw new InstantiationException("Configuration error: cannot create the CREAM working directories!");
            }
            
            if (service.getStringParameter(ServiceAttribute.DELEGATION_STORAGE_LABEL) == null) {
                throw new InstantiationException("Configuration error: " + 
                		ServiceAttribute.DELEGATION_STORAGE_LABEL + " property not defined!");
            }

            if (service.getStringParameter(ServiceAttribute.DELEGATION_FACTORY_LABEL) == null) {
                throw new InstantiationException("Configuration error: " + 
                		ServiceAttribute.DELEGATION_FACTORY_LABEL + " property not defined!");
            }

            getDelegationService();           

            try {
                delegationSuffix = DelegationManager.getInstance().getDelegationSuffix();
            } catch (Throwable t) {
                throw new InstantiationException("cannot get instance of DelegationManager: " + t.getMessage());
            }
            
            initialized = true;

            // Introspector.flushCaches();

            serviceInfo.setDoesAcceptNewJobSubmissions(true);
            serviceInfo.setStatus("RUNNING");
            
            logger.info("CREAM initialization done!");
        }
    }

    public Result[] insertCommand(JobFilter filter, JobCmd cmd) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        logger.debug("Begin insertCommand");

    	if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("insertCommand", "0", "filter not specified!", null);
        }

        if (cmd == null) {
            throw FaultFactory.makeInvalidArgumentFault("insertCommand", "0", "cmd not specified!", null);
        }

        ServiceConfig service = ServiceConfig.getConfiguration();
        if (service == null) {
            throw FaultFactory.makeGenericFault("jobRegister", "0", "system error", "Cannot access to the configuration");
        }
        
        String userId = getUserDN_FQAN(null);     
        //String authNProxyPath = service.getStringParameter(ServiceAttribute.JOB_DIR_LABEL) + File.separator + "user_proxy";
        //authNProxyPath += File.separator + userId;
        //storeAuthNProxy(authNProxyPath);

        if (logger.isDebugEnabled()) {
            // code for log information
            logger.debug("insertCommand: filter for command " + cmd.getName());

            JobId[] jobIdObjArray = filter.getJobId();
            if (jobIdObjArray != null) {
                for (int index = 0; index < jobIdObjArray.length; index++) {
                    logger.debug("" + index + ") jobId = " + jobIdObjArray[index].getCreamURL() + "/" + jobIdObjArray[index].getId());
                }
            }
            if (filter.getLeaseId() != null) {
                logger.debug("LeaseId = " + filter.getLeaseId());
            }
            if (filter.getDelegationId() != null) {
                logger.debug("DelegationId = " + filter.getDelegationId());
            }
            if (filter.getFromDate() != null) {
                logger.debug("FromDate = " + filter.getFromDate().getTime());
            }
            if (filter.getToDate() != null) {
                logger.debug("ToDate = " + filter.getToDate().getTime());
            }
            if (filter.getStatus() != null) {
                logger.debug("Status size = " + filter.getStatus().length);
            }
        }

        cmd.setUserId(userId);
        cmd.setIsAdmin(CEUtils.isAdmin());
        cmd.setUserDN(CEUtils.getUserDN_X500());
        cmd.setUserFQAN(CEUtils.getFQAN(null));
        cmd.setLocalUser(CEUtils.getLocalUser());
        cmd.setLocalUserGroup(CEUtils.getLocalUserGroup());
        cmd.setRemoteRequestAddress(CEUtils.getRemoteRequestAddress());
        cmd.setJobStatus(filter.getStatus());
        cmd.setFromDate(filter.getFromDate());
        cmd.setToDate(filter.getToDate());
        cmd.setDelegationProxyId(filter.getDelegationId());
        cmd.setLeaseId(filter.getLeaseId());

        JobId[] jobIds = filter.getJobId();
        if (jobIds != null) {
            List<String> jobIdList = new ArrayList<String>(0);
            for (int i=0; i<jobIds.length; i++) {
                jobIdList.add(jobIds[i].getId());
            }
            cmd.setJobIdList(jobIdList);
        }
        
        List<Result> resultList = new ArrayList<Result>(0);
        List<String> jobIdFound = null;

        if (cmd.isAsynchronous()) {
            if (jobIds != null && jobIds.length > 0) {
                String[] compatibleStatus = filter.getStatus();

                if (compatibleStatus == null) {
                    switch (cmd.getType()) {
                    case JobCommandConstant.JOB_CANCEL:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.REGISTERED), JobStatus.getNameByType(JobStatus.IDLE),
                                JobStatus.getNameByType(JobStatus.HELD), JobStatus.getNameByType(JobStatus.REALLY_RUNNING), JobStatus.getNameByType(JobStatus.RUNNING) };
                        break;
                    case JobCommandConstant.JOB_PURGE:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.ABORTED), JobStatus.getNameByType(JobStatus.REGISTERED),
                                JobStatus.getNameByType(JobStatus.CANCELLED), JobStatus.getNameByType(JobStatus.DONE_OK), JobStatus.getNameByType(JobStatus.DONE_FAILED) };
                        break;
                    case JobCommandConstant.JOB_RESUME:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.HELD) };
                        break;
                    case JobCommandConstant.JOB_SET_LEASEID:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.PENDING), JobStatus.getNameByType(JobStatus.IDLE),
                                JobStatus.getNameByType(JobStatus.HELD), JobStatus.getNameByType(JobStatus.REALLY_RUNNING), JobStatus.getNameByType(JobStatus.RUNNING) };
                        break;
                    case JobCommandConstant.JOB_SUSPEND:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.IDLE), JobStatus.getNameByType(JobStatus.REALLY_RUNNING),
                                JobStatus.getNameByType(JobStatus.RUNNING) };
                        break;
                    case JobCommandConstant.JOB_START:
                        compatibleStatus = new String[] { JobStatus.getNameByType(JobStatus.REGISTERED) };
                        break;
                    }
                }

                JobInfoCmd jfCmd = new JobInfoCmd();
                jfCmd.setUserId(cmd.getUserId());
                jfCmd.setJobStatus(compatibleStatus);
                jfCmd.setIsAdmin(cmd.isAdmin());
                jfCmd.setFromDate(filter.getFromDate());
                jfCmd.setToDate(filter.getToDate());
                jfCmd.setDelegationProxyId(filter.getDelegationId());
                jfCmd.setLeaseId(filter.getLeaseId());
                jfCmd.setJobIdList(cmd.getJobIdList());

                try {
                    logger.debug("Calling commandManager.insertCommand for synchronous command.");
                    CommandManager.getInstance().insertCommand(jfCmd);
                    logger.debug("CommandManager.getInstance().insertCommand for synchronous command executed.");

                    jobIdFound = jfCmd.getJobIdFound();

                    fillupResultList(resultList, jfCmd.getJobIdFilterResult(), cmd.getName());
                } catch (IllegalArgumentException e) {
                    logger.error("IllegalArgumentException: " + e.getMessage());
                    throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
                } catch (CommandException e) {
                    logger.error("CommandException: " + e.getMessage());
                    throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
                } catch (CommandManagementException e) {
                    logger.error("CommandManagementException: " + e.getMessage());
                    throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
                }
            }

            if (jobIdFound == null || jobIdFound.size() > 1) {
                cmd.addParameter("EXECUTION_MODE", cmd.getExecutionMode().getStringValue());
                cmd.addParameter("PRIORITY_LEVEL", "" + cmd.getPriorityLevel());
                cmd.setCommandGroupId("COMPOUND");
                cmd.setExecutionMode(ExecutionModeValues.PARALLEL);
                cmd.setPriorityLevel(JobCmd.MEDIUM_PRIORITY);                
            } else if (jobIdFound.size() == 1) {
                cmd.setCommandGroupId(jobIdFound.get(0));
            }
        }

        if (!cmd.isAsynchronous() || (jobIdFound == null || jobIdFound.size() > 0)) {
            try {
                logger.debug("Calling CommandManager.getInstance().insertCommand for synchronous command.");
                CommandManager.getInstance().insertCommand(cmd);
                logger.debug("CommandManager.getInstance().insertCommand for synchronous command executed.");
            } catch (Throwable e) {
                logger.error("insertCommand error: " + e.getMessage());
                throw FaultFactory.makeGenericFault("insertCommand", "1", e.getMessage(), e.getMessage());
            }
        }

        if (!cmd.isAsynchronous()) {
            fillupResultList(resultList, cmd.getJobIdFilterResult(), cmd.getName());
        }

        Result[] result = new Result[resultList.size()];
        result = resultList.toArray(result);

        logger.debug("End insertCommand");

        return result;
    }
    
    private void fillupResultList(List<Result> resultList, List<JobIdFilterResult> filterResultList, String commandName) {
        if (resultList == null || filterResultList == null) {
            return;
        }

        JobUnknownFault jobNotFoundFault = FaultFactory.makeJobUnknownFault(commandName, "1", "job not found!", "N/A");
        JobStatusInvalidFault jobStatusInvalidFault = FaultFactory.makeJobStatusInvalidFault(commandName, "2", "the job has a status not compatible with the " + commandName + " command!", "N/A");
        DateMismatchFault dateMismatchFault = FaultFactory.makeDateMismatchFault(commandName, "3", "fromDate/toDate mismatch", "N/A");
        DelegationIdMismatchFault delegationMismatchFault = FaultFactory.makeDelegationIdMismatchFault(commandName, "3", "delegationId mismatch", "N/A");
        LeaseIdMismatchFault leaseIdMismatchFault = FaultFactory.makeLeaseIdMismatchFault(commandName, "3", "leaseId mismatch", "N/A");

        Result result = null;

        for (JobIdFilterResult filterResult : filterResultList) {
            result = new Result();
            result.setJobId(new JobId(filterResult.getJobId(), creamURL, null));

            switch (filterResult.getErrorCode()) {
            case JobIdFilterFailure.OK_ERRORCODE:
                resultList.add(result);
                break;
            case JobIdFilterFailure.JOBID_ERRORCODE:
                result.setJobUnknownFault(jobNotFoundFault);
                resultList.add(result);
                break;
            case JobIdFilterFailure.STATUS_ERRORCODE:
                result.setJobStatusInvalidFault(jobStatusInvalidFault);
                resultList.add(result);
                break;
            case JobIdFilterFailure.LEASEID_ERRORCODE:
                result.setLeaseIdMismatchFault(leaseIdMismatchFault);
                resultList.add(result);
                break;
            case JobIdFilterFailure.DATE_ERRORCODE:
                result.setDateMismatchFault(dateMismatchFault);
                resultList.add(result);
                break;
            case JobIdFilterFailure.DELEGATIONID_ERRORCODE:
                result.setDelegationIdMismatchFault(delegationMismatchFault);
                resultList.add(result);
                break;
            }
        }
    }
    
    public Result[] jobCancel(JobFilter filter) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobCancel", "0", "filter not specified!", null);
        }
        return insertCommand(filter, new JobCancelCmd());
    }

    public JobInfoResult[] jobInfo(JobFilter filter) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobInfo", "0", "filter not specified!", null);
        }
        
        JobInfoCmd cmd = new JobInfoCmd();
        insertCommand(filter, cmd);
        
        JobEnumeration jobEnum = cmd.getJobEnumeration();
        List<JobIdFilterResult> jobFilterResult = cmd.getJobIdFilterResult();
                
        String creamBaseUrl = "";
        if (creamURL != null) {
        	logger.debug("creamURL = " + creamURL.toString());
        	int serviceStringIndex = creamURL.toString().lastIndexOf("ce-cream/services");
        	if (serviceStringIndex != -1) {
        		creamBaseUrl = creamURL.toString().substring(0, serviceStringIndex);
        	}
        }
        
        JobInfoResult[] result = new JobInfoResult[jobFilterResult.size()];
        int index = 0;
        
        if (jobEnum != null) {
            boolean isAdmin = CEUtils.isAdmin();
            Job job = null;
            
            while (jobEnum.hasMoreJobs()) {
                try {
                    job = jobEnum.nextJob();
                    result[index] = new JobInfoResult();
                    result[index].setJobDescriptionId(creamBaseUrl + job.getId());
                    result[index++].setJobInfo(makeJob(job, isAdmin));
                } catch (JobManagementException e) {
                    result[index] = new JobInfoResult();
                    result[index].setJobUnknownFault(FaultFactory.makeJobUnknownFault("jobInfo", "0", "job " + job.getId() + " retrieving error", e.getMessage()));
                    result[index++].setJobDescriptionId(creamBaseUrl + job.getId());
                }
            }
        }

        for (int i = 0; i < jobFilterResult.size(); i++) {
            switch (jobFilterResult.get(i).getErrorCode()) {
            case JobIdFilterFailure.OK_ERRORCODE:
                break;
            case JobIdFilterFailure.DATE_ERRORCODE:
                result[index] = new JobInfoResult();
                result[index].setJobDescriptionId(creamBaseUrl + jobFilterResult.get(i).getJobId());
                result[index++].setDateMismatchFault(FaultFactory.makeDateMismatchFault("jobInfo", "0", "fromDate/toDate mismatch", "N/A"));
                break;
            case JobIdFilterFailure.DELEGATIONID_ERRORCODE:
                result[index] = new JobInfoResult();
                result[index].setJobDescriptionId(creamBaseUrl + jobFilterResult.get(i).getJobId());
                result[index++].setDelegationIdMismatchFault(FaultFactory.makeDelegationIdMismatchFault("jobInfo", "1", "delegationId mismatch", "N/A"));
                break;
            case JobIdFilterFailure.JOBID_ERRORCODE:
                result[index] = new JobInfoResult();
                result[index].setJobDescriptionId(creamBaseUrl + jobFilterResult.get(i).getJobId());
                result[index++].setJobUnknownFault(FaultFactory.makeJobUnknownFault("jobInfo", "2", "job not found", "N/A"));
                logger.debug("JobInfo method: job not found (jobId = " + jobFilterResult.get(i).getJobId() +
                		      " userId = " + cmd.getUserId() + ")");
                break;
            case JobIdFilterFailure.LEASEID_ERRORCODE:
                result[index] = new JobInfoResult();
                result[index].setJobDescriptionId(creamBaseUrl + jobFilterResult.get(i).getJobId());
                result[index++].setLeaseIdMismatchFault(FaultFactory.makeLeaseIdMismatchFault("jobInfo", "3", "leaseId mismatch", "N/A"));
                break;
            case JobIdFilterFailure.STATUS_ERRORCODE:
                result[index] = new JobInfoResult();
                result[index].setJobDescriptionId(creamBaseUrl + jobFilterResult.get(i).getJobId());
                result[index++].setJobStatusInvalidFault(FaultFactory.makeJobStatusInvalidFault("jobInfo", "4", "job status mismatch", "N/A"));
                break;
            } 
        }
        
        return result;
    }    
    
    public Result[] jobSetLeaseId(JobSetLeaseIdRequest req) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (req.getJobFilter() == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobSetLeaseId", "0", "job filter not specified!", null);
        }
        return insertCommand(req.getJobFilter(), new JobSetLeaseIdCmd(req.getNewLeaseId()));
    }

    public Lease[] getLeaseList() throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        GetLeaseCmd cmd = new GetLeaseCmd();
        cmd.setUserId(getUserDN_FQAN(null));
        cmd.setIsAdmin(CEUtils.isAdmin());
        
        try {
            CommandManager.getInstance().insertCommand(cmd);
            List<org.glite.ce.creamapi.jobmanagement.Lease> leaseList = cmd.getLease();
            if (leaseList != null) {
                Lease[] leaseArray = new Lease[leaseList.size()];
                for (int i=0; i<leaseList.size(); i++) {
                    leaseArray[i] = new Lease();
                    leaseArray[i].setLeaseId(leaseList.get(i).getLeaseId());
                    leaseArray[i].setLeaseTime(leaseList.get(i).getLeaseTime());
                }
                return leaseArray;
            } else {
                return new Lease[0];
            }
        } catch (IllegalArgumentException e) {
            throw FaultFactory.makeGenericFault("getLeaseList", "0", "IllegalArgumentException occured", e.getMessage());
        } catch (CommandException e) {
            throw FaultFactory.makeGenericFault("getLeaseList", "1", "CommandException occured", e.getMessage());
        } catch (CommandManagementException e) {
            throw FaultFactory.makeGenericFault("getLeaseList", "2", "CommandManagementException occured", e.getMessage());
        }
    }

    public Lease getLease(String leaseId) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (leaseId == null) {
            throw FaultFactory.makeInvalidArgumentFault("getLease", "0", "leaseId not specified!", null);
        }

        GetLeaseCmd cmd = new GetLeaseCmd(leaseId);
        cmd.setUserId(getUserDN_FQAN(null));
        cmd.setIsAdmin(CEUtils.isAdmin());
        
        try {
            CommandManager.getInstance().insertCommand(cmd);
            List<org.glite.ce.creamapi.jobmanagement.Lease> leaseList = cmd.getLease();
            if (leaseList != null && leaseList.size() > 0) {
                Lease lease = new Lease();
                lease.setLeaseId(leaseList.get(0).getLeaseId());
                lease.setLeaseTime(leaseList.get(0).getLeaseTime());
                return lease;
            } else {
                throw FaultFactory.makeGenericFault("getLease", "1", "lease " + leaseId + " not found!", "N/A");
            }
        } catch (IllegalArgumentException e) {
            throw FaultFactory.makeGenericFault("getLease", "0", "IllegalArgumentException occured", e.getMessage());
        } catch (CommandException e) {
            throw FaultFactory.makeGenericFault("getLease", "1", "CommandException occured", e.getMessage());
        } catch (CommandManagementException e) {
            throw FaultFactory.makeGenericFault("getLease", "2", "CommandManagementException occured", e.getMessage());
        }
    }

    public void deleteLease(String leaseId) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (leaseId == null) {
            throw FaultFactory.makeInvalidArgumentFault("deleteLease", "0", "leaseId not specified!", null);
        }
        
        DeleteLeaseCmd cmd = new DeleteLeaseCmd(leaseId);
        cmd.setUserId(getUserDN_FQAN(null));
        cmd.setIsAdmin(CEUtils.isAdmin());
        
        try {
            CommandManager.getInstance().insertCommand(cmd);
        } catch (IllegalArgumentException e) {
            throw FaultFactory.makeGenericFault("deleteLease", "0", "IllegalArgumentException occured", e.getMessage());
        } catch (CommandException e) {
            throw FaultFactory.makeGenericFault("deleteLease", "1", "CommandException occured", e.getMessage());
        } catch (CommandManagementException e) {
            throw FaultFactory.makeGenericFault("deleteLease", "2", "CommandManagementException occured", e.getMessage());
        }
    }
    
    public Lease setLease(Lease lease) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (lease == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobLease", "0", "lease not specified!", null);
        }
        if (lease.getLeaseId() == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobLease", "0", "leaseId not specified!", null);
        }
        
        SetLeaseCmd cmd = new SetLeaseCmd(lease.getLeaseId(), lease.getLeaseTime());
        cmd.setUserId(getUserDN_FQAN(null));
        cmd.setIsAdmin(CEUtils.isAdmin());
        
        try {
            CommandManager.getInstance().insertCommand(cmd);
            
            lease.setLeaseTime(cmd.getLeaseTimeResult());
            
            return lease;
        } catch (IllegalArgumentException e) {
            throw FaultFactory.makeGenericFault("jobLease", "0", "IllegalArgumentException occured", e.getMessage());
        } catch (CommandException e) {
            throw FaultFactory.makeGenericFault("jobLease", "1", "CommandException occured", e.getMessage());
        } catch (CommandManagementException e) {
            throw FaultFactory.makeGenericFault("jobLease", "2", "CommandManagementException occured", e.getMessage());
        }
    }
    
    public JobId[] jobList() throws RemoteException, GenericFault, AuthorizationFault {
        try {
            JobListCmd cmd = new JobListCmd();
            cmd.setUserId(getUserDN_FQAN(null));
            cmd.setIsAdmin(CEUtils.isAdmin());

            CommandManager.getInstance().insertCommand(cmd);

            List<String> list = cmd.getJobIdFound();
            if (list != null) {
                JobId[] idList = new JobId[list.size()];

                for (int i = 0; i < list.size(); i++) {
                    idList[i] = new JobId();
                    idList[i].setCreamURL(creamURL);
                    idList[i].setId((String) list.get(i));
                }

                return idList;
            }
        } catch (CommandException e) {
            throw FaultFactory.makeGenericFault("jobList", "0", "system error", e.getMessage());
        } catch (CommandManagementException e) {
            throw FaultFactory.makeGenericFault("jobList", "1", "CommandManagementException occured", e.getMessage());
        }

        return new JobId[0];
    }

    public Result[] jobPurge(JobFilter filter) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobPurge", "0", "filter not specified!", null);
        }

        return insertCommand(filter, new JobPurgeCmd());
    }

    public JobRegisterResult[] jobRegister(JobDescription[] descr) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault, JobSubmissionDisabledFault {
        if (descr == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobRegister", "0", "filter not specified!", null);
        }

        JobSubmissionManagerInfo jobSubmissionManagerInfo = JobSubmissionManager.getInstance().getJobSubmissionManagerInfo();

        if (!jobSubmissionManagerInfo.isAcceptNewJobs()) {
            String acceptNewJobSubmissionErrorMessage = "Submissions are disabled!";
            if ((jobSubmissionManagerInfo.getTestErrorMessage() != null)) {
               acceptNewJobSubmissionErrorMessage = jobSubmissionManagerInfo.getTestErrorMessage();
            }

            JobSubmissionDisabledFault fault = FaultFactory.makeJobSubmissionDisabledFault("jobRegister", "0", "The CREAM service cannot accept jobs at the moment", acceptNewJobSubmissionErrorMessage);
            throw (fault);            
        }

        ServiceConfig service = ServiceConfig.getConfiguration();
        if (service == null) {
            throw FaultFactory.makeGenericFault("jobRegister", "0", "system error", "Cannot access to the configuration");
        }

        //System.out.println(org.apache.axis.utils.XMLUtils.ElementToString(messageContext.getRequestMessage().getSOAPEnvelope()));
        List<String> fqanlist = null;
        
        String userVO = CEUtils.getUserDefaultVO();
        if (userVO != null) {
            fqanlist = CEUtils.getFQAN(userVO);
        }
        
        String fqan = "";
        if (fqanlist != null && fqanlist.size() > 0) {
            fqan = fqanlist.get(0).toString();
        }
        
        String userId = normalize(CEUtils.getUserDN_X500() + fqan);
        String iceId = null;
        
        SOAPHeaderElement headerElement = MessageContext.getCurrentContext().getRequestMessage().getSOAPEnvelope().getHeaderByName("http://glite.org/2007/11/ce/cream/types", "iceId");            
        
        if (headerElement == null) {       
            headerElement = MessageContext.getCurrentContext().getRequestMessage().getSOAPEnvelope().getHeaderByName("http://glite.org/ce/creamapij/types", "iceId");
        }
        
        if (headerElement != null && headerElement.getValue() != null && headerElement.getValue().length() > 0) {
            iceId = headerElement.getValue();
            headerElement.setProcessed(true);
        }
        
       // String authNProxyPath = service.getStringParameter(ServiceAttribute.JOB_DIR_LABEL) + File.separator + "user_proxy" + File.separator + userId + File.separator + "authN.proxy";
        
        JobRegisterResult[] result = new JobRegisterResult[descr.length];
        Hashtable<String, Delegation> delegationIdtable = new Hashtable<String, Delegation>(0);
        
        for (int i = 0; i < descr.length; i++) {
            result[i] = new JobRegisterResult();
            result[i].setJobDescriptionId(descr[i].getJobDescriptionId());
            try {
                String delegationId = descr[i].getDelegationId();
                String delegationProxy = descr[i].getDelegationProxy();

                if (delegationId == null) {
                    result[i].setDelegationIdMismatchFault(FaultFactory.makeDelegationIdMismatchFault("jobRegister", "0", "delegation id not specified!", "N/A"));
                    continue;
                }

                Delegation delegation = null;

                if (!delegationIdtable.containsKey(delegationId)) {
                    if (delegationProxy != null && delegationProxy.length() > 0) {
                        try {
                            getDelegationService().putProxy(delegationId, delegationProxy); 
                        } catch (DelegationException e) {
                            result[i].setDelegationProxyFault(FaultFactory.makeDelegationProxyFault("jobRegister", "0", "delegation error: put proxy failed!", e.getMsg()));
                            continue;
                        }
                    }

                    try {
                        delegation = DelegationManager.getInstance().getDelegation(delegationId, CEUtils.getUserDN_X500(), CEUtils.getLocalUser());
                        //delegProxyInfo = getProxy(delegationId, CEUtils.getUserDN_X500(), CEUtils.getLocalUser());
                    } catch (Exception e) {
                        result[i].setDelegationIdMismatchFault(FaultFactory.makeDelegationIdMismatchFault("jobRegister", "0", "delegation error: delegation id \"" + delegationId + "\" not found!", e.getMessage()));
                        continue;
                    }
                    
                    delegationIdtable.put(delegationId, delegation);
                } else {
                    delegation = delegationIdtable.get(delegationId);
                }
                
                if (delegation == null) {
                    String message = "delegation error: proxy delegationID \"" + delegationId + "\" not found!;";
                    result[i].setDelegationProxyFault(FaultFactory.makeDelegationProxyFault("jobRegister", "0", message, "delegation proxy not found!"));
                    continue;
                }

                if (!delegation.isValid()) {
                    String message = "delegation error: the proxy delegationID \"" + delegationId + "\" is not more valid!;";
                    result[i].setDelegationProxyFault(FaultFactory.makeDelegationProxyFault("jobRegister", "0", message, "delegation proxy expired!"));
                    continue;
                }

                JobRegisterCmd cmd = new JobRegisterCmd();
                cmd.setLocalUser(CEUtils.getLocalUser());
                cmd.setLocalUserGroup(CEUtils.getLocalUserGroup());
                cmd.setUserId(userId);
                cmd.setUserDN(CEUtils.getUserDN_X500());
                cmd.setUserFQAN(fqanlist);
                cmd.setUserVO(CEUtils.getUserDefaultVO());
                cmd.setJDL(descr[i].getJDL());
                cmd.setAutostart(false);
                // cmd.setAutostart(descr[i].isAutoStart());
//                cmd.setAuthNProxyPath(authNProxyPath);
                cmd.setDelegationProxyId(descr[i].getDelegationId());
                cmd.setDelegationProxySuffix(delegationSuffix);
                //cmd.setDelegationProxyPath(sandboxDir + File.separator + cmd.getLocalUserGroup() + File.separator + userId + "_" + cmd.getLocalUser() + File.separator + "proxy" + File.separator + normalize(delegProxyInfo.getId()) + delegationSuffix);
                cmd.setDelegationProxyInfo(delegation.toString());
                cmd.setLeaseId(descr[i].getLeaseId());
                cmd.setCREAMURL(creamURL.toString());
              //  cmd.setCEId(ceId);
                cmd.setICEId(iceId);
                cmd.setAsynchronous(false);
                cmd.setRemoteRequestAddress(CEUtils.getRemoteRequestAddress());
                
                String url = creamURL.toString();
                String gsiFTPcreamURL = url.substring(url.indexOf(":"), url.lastIndexOf(":"));
                cmd.setGSIFTPCREAMURL("gsiftp" + gsiFTPcreamURL);
                
                CommandManager.getInstance().insertCommand(cmd);

                Job job = cmd.getJob();

                if (job == null) {
                    result[i].setGenericFault(FaultFactory.makeGenericFault("jobRegister", "0", "job not created!", null));
                    continue;
                }

                if (descr[i].isAutoStart() && job.getStatusCount() == 1 && job.getLastStatus().getType() == JobStatus.REGISTERED) {
                    JobStartCmd jobStartCmd = new JobStartCmd();
                    jobStartCmd.setUserId(userId);
                    jobStartCmd.setUserDN(cmd.getUserDN());
                    jobStartCmd.setUserFQAN(cmd.getUserFQAN());
                    jobStartCmd.setLocalUser(CEUtils.getLocalUser());
                    jobStartCmd.setLocalUserGroup(CEUtils.getLocalUserGroup());
                    jobStartCmd.setJobId(job.getId());
                    jobStartCmd.setCommandGroupId(job.getId());
                    jobStartCmd.setRemoteRequestAddress(cmd.getRemoteRequestAddress());

                    CommandManager.getInstance().insertCommand(jobStartCmd);
                }

                JobId jobId = new JobId();
                jobId.setCreamURL(creamURL);
                jobId.setId(job.getId());
                
                if (job.getCREAMInputSandboxURI() != null) {
                    org.glite.ce.creamapi.ws.cream2.types.Property[] property = new org.glite.ce.creamapi.ws.cream2.types.Property[((iceId == null)? 2: 3)];
                    property[0] = new org.glite.ce.creamapi.ws.cream2.types.Property();
                    property[0].setName("CREAMInputSandboxURI");
                    property[0].setValue(job.getCREAMInputSandboxURI());
                    property[1] = new org.glite.ce.creamapi.ws.cream2.types.Property();
                    property[1].setName("CREAMOutputSandboxURI");
                    property[1].setValue(job.getCREAMOutputSandboxURI());
                    
                    if (iceId != null) {
                        property[2] = new org.glite.ce.creamapi.ws.cream2.types.Property();
                        property[2].setName("DB_ID");
                        property[2].setValue(""+dbID);
                    }

                    jobId.setProperty(property);
                }

                result[i].setJobId(jobId);      
            } catch (GenericFault e) {
                result[i].setGenericFault(e);
            } catch (Throwable e) {
                if (e.getMessage().startsWith("lease id")) {
                    result[i].setLeaseIdMismatchFault(FaultFactory.makeLeaseIdMismatchFault("jobRegister", "0", "lease id not found!", e.getMessage()));
                } else {
                    result[i].setGenericFault(FaultFactory.makeGenericFault("jobRegister", "0", "system error", e.getMessage()));
                }
            }
        }

        return result;
    }

    public Result[] jobResume(JobFilter filter) throws RemoteException, OperationNotSupportedFault, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobResume", "0", "filter not specified!", null);
        }
        return insertCommand(filter, new JobResumeCmd());
    }

    public Result[] jobStart(JobFilter filter) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
        logger.debug("Begin jobStart");
        
    	if (filter == null) {
    		logger.error("filter not specified!");
            throw FaultFactory.makeInvalidArgumentFault("jobStart", "0", "filter not specified!", null);
        }
    	
        return insertCommand(filter, new JobStartCmd());
    }

    public JobStatusResult[] jobStatus(JobFilter filter) throws RemoteException, AuthorizationFault, GenericFault, InvalidArgumentFault {
    	logger.debug("Begin jobStatus");
    	
    	if (filter == null) {
    		logger.error("filter not specified!");
            throw FaultFactory.makeInvalidArgumentFault("jobStatus", "0", "filter not specified!", null);
        }

        int index = 0;
        JobStatusCmd cmd = new JobStatusCmd();
        Result[] result = insertCommand(filter, cmd);

        List<JobStatus> statusList = cmd.getJobStatusResult();
        List<JobIdFilterResult> jobIdFilterList = cmd.getJobIdFilterResult();

        String creamBaseUrl = "";
        if (creamURL != null) {
            logger.debug("creamURL = " + creamURL.toString());
            int serviceStringIndex = creamURL.toString().lastIndexOf("ce-cream/services");
            if (serviceStringIndex != -1) {
                creamBaseUrl = creamURL.toString().substring(0, serviceStringIndex);
            }
        }
        
        JobStatusResult[] jsResult = null;
        
        if (filter.getJobId() == null || filter.getJobId().length == 0) {
            jsResult = new JobStatusResult[statusList.size()];
        } else {
            jsResult = new JobStatusResult[jobIdFilterList.size()];
            
            for (JobIdFilterResult filterResult : jobIdFilterList) {
                switch (filterResult.getErrorCode()) {
                case JobIdFilterFailure.OK_ERRORCODE:
                    break;
                case JobIdFilterFailure.DATE_ERRORCODE:
                    jsResult[index] = new JobStatusResult();
                    jsResult[index].setJobDescriptionId(creamBaseUrl + filterResult.getJobId());
                    jsResult[index++].setDateMismatchFault(FaultFactory.makeDateMismatchFault("jobStatus", "0", "fromDate/toDate mismatch", "N/A"));
                    break;
                case JobIdFilterFailure.DELEGATIONID_ERRORCODE:
                    jsResult[index] = new JobStatusResult();
                    jsResult[index].setJobDescriptionId(creamBaseUrl + filterResult.getJobId());
                    jsResult[index++].setDelegationIdMismatchFault(FaultFactory.makeDelegationIdMismatchFault("jobStatus", "1", "delegationId mismatch", "N/A"));
                    break;
                case JobIdFilterFailure.JOBID_ERRORCODE:
                    jsResult[index] = new JobStatusResult();
                    jsResult[index].setJobDescriptionId(creamBaseUrl + filterResult.getJobId());
                    jsResult[index++].setJobUnknownFault(FaultFactory.makeJobUnknownFault("jobStatus", "2", "job not found", "N/A"));
                    break;
                case JobIdFilterFailure.LEASEID_ERRORCODE:
                    jsResult[index] = new JobStatusResult();
                    jsResult[index].setJobDescriptionId(creamBaseUrl + filterResult.getJobId());
                    jsResult[index++].setLeaseIdMismatchFault(FaultFactory.makeLeaseIdMismatchFault("jobStatus", "3", "leaseId mismatch", "N/A"));
                    break;
                case JobIdFilterFailure.STATUS_ERRORCODE:
                    jsResult[index] = new JobStatusResult();
                    jsResult[index].setJobDescriptionId(creamBaseUrl + filterResult.getJobId());
                    jsResult[index++].setJobStatusInvalidFault(FaultFactory.makeJobStatusInvalidFault("jobStatus", "4", "job status mismatch", "N/A"));
                    break;
                }    
            }
        }

        if (statusList != null && statusList.size() > 0) {
            for (JobStatus jobStatus : statusList) {
                JobId jobId = new JobId();
                jobId.setId(jobStatus.getJobId());
                jobId.setCreamURL(creamURL);

                Status status = new Status();
                status.setDescription(jobStatus.getDescription());
                status.setExitCode(jobStatus.getExitCode());
                status.setFailureReason(jobStatus.getFailureReason());
                status.setJobId(jobId);
                status.setName(jobStatus.getName());
                status.setTimestamp(jobStatus.getTimestamp());

                jsResult[index] = new JobStatusResult();
                jsResult[index].setJobDescriptionId(creamBaseUrl + jobStatus.getJobId());
                jsResult[index++].setJobStatus(status);
            }
        }
        
        logger.debug("End jobStatus");
        
        return jsResult;
    }

    public Result[] jobSuspend(JobFilter filter) throws RemoteException, OperationNotSupportedFault, AuthorizationFault, GenericFault, InvalidArgumentFault {
        if (filter == null) {
            throw FaultFactory.makeInvalidArgumentFault("jobSuspend", "0", "filter not specified!", null);
        }        
        return insertCommand(filter, new JobSuspendCmd());
    }

    private Class makeCommandExecutorInstance(String jarFilePath) throws IllegalArgumentException {
        if (jarFilePath == null) {
            throw (new IllegalArgumentException("jarFilePath not specified!"));
        }

        File jarFile = new File(jarFilePath);
        try {
            if (!jarFile.getName().endsWith(".jar")) {
                throw (new RuntimeException("Cannot create a CommandExecutor instance from file \"" + jarFile.getAbsolutePath() + "\""));
            }

            URL connectorURL = jarFile.toURI().toURL();

            JarClassLoader loader = new JarClassLoader("file:" + jarFile.getPath(), this.getClass().getClassLoader());
            loader.addJarURL(connectorURL);

            String className = loader.getMainClassName(connectorURL);
            if (className != null) {
                return loader.loadClass(className);
            } else {
                throw (new RuntimeException("\"Main-Class\" attribute not found into the MANIFEST.MF of " + jarFile.getAbsolutePath() + ".jar"));
            }
        } catch (Exception e) {
            throw new RuntimeException("Cannot create the CommandExecutor class from file " + jarFile.getName() + ". Reason: " + jarFilePath + " " + e.getMessage());
        } finally {
            jarFile = null;
        }
    }

    private JobInfo makeJob(Job job, boolean isAdmin) {
        if (job == null) {
            return null;
        }
        
        JobId id = new JobId();
        id.setId(job.getId());
        
        try {
            id.setCreamURL(new URI(job.getCreamURL()));
        } catch (MalformedURIException e) {}
        
        JobInfo jobInfo = new JobInfo();
        jobInfo.setJobId(id);
        
        if (job.getCREAMInputSandboxURI() != null) {
            jobInfo.setCREAMInputSandboxURI(job.getCREAMInputSandboxURI());            
        } else {
            jobInfo.setCREAMInputSandboxURI(Job.NOT_AVAILABLE_VALUE);
        }

        if (job.getCREAMOutputSandboxURI() != null) {
            jobInfo.setCREAMOutputSandboxURI(job.getCREAMOutputSandboxURI()); 
        } else {
            jobInfo.setCREAMOutputSandboxURI(Job.NOT_AVAILABLE_VALUE);
        }

        if (job.getDelegationProxyInfo() != null) {
            jobInfo.setDelegationProxyInfo(job.getDelegationProxyInfo());            
        } else {
            jobInfo.setDelegationProxyInfo(Job.NOT_AVAILABLE_VALUE);
        }

        if (job.getGridJobId() != null) {
            jobInfo.setGridJobId(job.getGridJobId());
        } else {
            jobInfo.setGridJobId(Job.NOT_AVAILABLE_VALUE);
        }

        if (job.getLocalUser() != null) {
            jobInfo.setLocalUser(job.getLocalUser());
        } else {
            jobInfo.setLocalUser(Job.NOT_AVAILABLE_VALUE);
        }

        if (isAdmin) {
            if (job.getLRMSAbsLayerJobId() != null) {
                jobInfo.setLRMSAbsLayerJobId(job.getLRMSAbsLayerJobId());            
            } else {
                jobInfo.setLRMSAbsLayerJobId(Job.NOT_AVAILABLE_VALUE);
            }
        } else {
            jobInfo.setLRMSAbsLayerJobId("[reserved]");
        }

        if (isAdmin) {
            if (job.getLRMSJobId() != null) {
                jobInfo.setLRMSJobId(job.getLRMSJobId());            
            } else {
                jobInfo.setLRMSJobId(Job.NOT_AVAILABLE_VALUE);
            }
        } else {
            jobInfo.setLRMSJobId("[reserved]");
        }

        if (isAdmin) {
            if (job.getWorkingDirectory() != null) {
                jobInfo.setWorkingDirectory(job.getWorkingDirectory());
            } else {
                jobInfo.setWorkingDirectory(Job.NOT_AVAILABLE_VALUE);
            }
        } else {
            jobInfo.setWorkingDirectory("[reserved]");
        }

        if (job.getWorkerNode() != null) {
            jobInfo.setWorkerNode(job.getWorkerNode());
        } else {
            jobInfo.setWorkerNode(Job.NOT_AVAILABLE_VALUE);
        }
        
        if (job.getType() != null) {
            jobInfo.setType(job.getType());
        } else {
            jobInfo.setType(Job.NOT_AVAILABLE_VALUE);
        }
        
        jobInfo.setDelegationProxyId(job.getDelegationProxyId());
        jobInfo.setJDL(job.getJDL());

        List<JobCommand> commandHistory = job.getCommandHistory();
        if (commandHistory != null) {
            Command[] cmd = new Command[commandHistory.size()];

            for (int i = 0; i < commandHistory.size(); i++) {
                JobCommand jobCmd = commandHistory.get(i);

                cmd[i] = new Command();
                cmd[i].setCategory("JOB_MANAGEMENT");
                cmd[i].setCreationTime(jobCmd.getCreationTime());
                cmd[i].setExecutionCompletedTime(jobCmd.getExecutionCompletedTime());
                cmd[i].setStartProcessingTime(jobCmd.getStartProcessingTime());
                cmd[i].setStartSchedulingTime(jobCmd.getStartSchedulingTime());
                cmd[i].setDescription(jobCmd.getDescription());
                cmd[i].setFailureReason(jobCmd.getFailureReason());
                cmd[i].setName(jobCmd.getName());
                cmd[i].setStatus(jobCmd.getStatusName());
            }

            jobInfo.setLastCommand(cmd);
        }

        List<JobStatus> statusHistory = job.getStatusHistory();

        if (statusHistory != null) {
            Status[] status = new Status[statusHistory.size()];
            
            for (int i = 0; i < statusHistory.size(); i++) {
                JobStatus jobStatus = statusHistory.get(i);

                status[i] = new Status();
                status[i].setDescription(jobStatus.getDescription());
                status[i].setExitCode(jobStatus.getExitCode());
                status[i].setFailureReason(jobStatus.getFailureReason());
                status[i].setJobId(id);
                status[i].setName(jobStatus.getName());
                status[i].setTimestamp(jobStatus.getTimestamp());
            }

            jobInfo.setStatus(status);
        }

        if (job.getLease() != null) {
            Lease lease = new Lease();
            lease.setLeaseId(job.getLease().getLeaseId());
            lease.setLeaseTime(job.getLease().getLeaseTime());

            jobInfo.setLease(lease);
        }

        return jobInfo;
    }

    protected String normalize(String s) {
        if (s != null) {
            return s.replaceAll("\\W", "_");
        }
        return null;
    }

    public ServiceInfo getServiceInfo(int verbosityLevel) throws RemoteException, GenericFault {
        List<org.glite.ce.creamapi.ws.cream2.types.Property> propertyList = new ArrayList<org.glite.ce.creamapi.ws.cream2.types.Property>(0);

        ServiceConfig service = ServiceConfig.getConfiguration();
        String cemonURL = service.getStringParameter(ServiceAttribute.CEMON_URL_LABEL);
        if (cemonURL != null) {
            propertyList.add(new org.glite.ce.creamapi.ws.cream2.types.Property("cemon_url", cemonURL));
        }

        JobSubmissionManagerInfo jobSubmissionManagerInfo = JobSubmissionManager.getInstance().getJobSubmissionManagerInfo();
        serviceInfo.setDoesAcceptNewJobSubmissions(jobSubmissionManagerInfo.isAcceptNewJobs());
        
        if (verbosityLevel >= 2) {
            try {
                GetServiceInfo getServiceInfoCmd = new GetServiceInfo();
                getServiceInfoCmd.setUserId(getUserDN_FQAN(null));
                CommandManager.getInstance().insertCommand(getServiceInfoCmd);
                serviceInfo.setDoesAcceptNewJobSubmissions(getServiceInfoCmd.doesAcceptNewJobs());

                String submissionThresholdMessage = getServiceInfoCmd.getResult().getParameterAsString("SUBMISSION_THRESHOLD_MESSAGE");
                if (submissionThresholdMessage != null) {
                    propertyList.add(new org.glite.ce.creamapi.ws.cream2.types.Property("SUBMISSION_THRESHOLD_MESSAGE", submissionThresholdMessage));
                }

                String submissionErrorMessage = getServiceInfoCmd.getResult().getParameterAsString("SUBMISSION_ERROR_MESSAGE");
                ServiceMessage[] serviceMessages = null;

                if (submissionErrorMessage != null) {
                    serviceMessages = new ServiceMessage[1];
                    serviceMessages[0] = new ServiceMessage();
                    serviceMessages[0].setMessage(submissionErrorMessage);
                    serviceMessages[0].setTimastamp((Calendar) getServiceInfoCmd.getResult().getParameter("SUBMISSION_EXECUTION_TIMESTAMP"));
                    serviceMessages[0].setType("SUBMISSION_ERROR_MESSAGE");
                }
                serviceInfo.setMessage(serviceMessages);
            } catch (Throwable e) {
                throw FaultFactory.makeGenericFault("getServiceInfo", "0", e.getMessage(), "N/A");
            }
        }

        if (propertyList.size() > 0) {
            org.glite.ce.creamapi.ws.cream2.types.Property[] properties = new org.glite.ce.creamapi.ws.cream2.types.Property[propertyList.size()];
            properties = propertyList.toArray(properties);
            serviceInfo.setProperty(properties);
        } else {
            serviceInfo.setProperty(null);
        }

        return serviceInfo;
    }

    private String makeLimitedDelegationProxy(DelegationProxyInfo delegProxyInfo) throws IllegalArgumentException, IOException {
        if (delegProxyInfo == null) {
            throw new IllegalArgumentException("delegProxyInfo not specified!");
        }
        
        if (proxyDir == null) {
            throw new IllegalArgumentException("proxyDir not defined!");
        }

        String normalizedId = normalize(delegProxyInfo.getId());
        String proxyPath = proxyDir + File.separator + delegProxyInfo.getUserId();                    

        File delegCertFile = new File(proxyPath);

        if (!delegCertFile.exists()) {
            File dir = new File(proxyPath);
            if (!dir.exists()) {
                dir.mkdirs();
            }
            dir = null;
        }

        proxyPath += File.separator + normalizedId + delegationSuffix;

        delegCertFile = new File(proxyPath);
        try {
            CEUtils.makeFile(delegCertFile, delegProxyInfo.getProxy(), false, true);
        } catch (OverlappingFileLockException e) {
        } catch (IOException e) {
             throw e;
        }

        Process proc = null;
        BufferedReader readIn = null;
        BufferedReader readErr = null;
        int secondsLeft = 0; 
        String[] gridProxyInit_CMD = null;
               
        try {
            proc = Runtime.getRuntime().exec(new String[] {"grid-proxy-info", "-timeleft", "-f", proxyPath });

            readIn = new BufferedReader(new InputStreamReader(proc.getInputStream()));
            readErr = new BufferedReader(new InputStreamReader(proc.getErrorStream()));

            if (proc.waitFor() == 0) {
                secondsLeft = Integer.parseInt(readIn.readLine());
                
                gridProxyInit_CMD = new String[] {"grid-proxy-init", "-old", "-limited", "-valid", (secondsLeft/3600) + ":" + (((secondsLeft%3600)/60) + 1),
                        "-cert", proxyPath, "-key", proxyPath, "-out", proxyPath + ".lmt" };
            } else {
                logger.error("storeDelegationProxy error: execution of 'grid-proxy-info -timeleft -f " + proxyPath +
                        "' failed (msgError = " + readErr.readLine() + "); using 'grid-proxy-init -old -limited -cert " + proxyPath +
                        " -key " + proxyPath + " -out " + proxyPath + ".lmt'");
                
                gridProxyInit_CMD = new String[] {"grid-proxy-init", "-old", "-limited",
                        "-cert", proxyPath, "-key", proxyPath, "-out", proxyPath + ".lmt" };
            }
        } catch (IOException ioe) {
            logger.error(ioe.getMessage(), ioe);
            delegCertFile.delete();
        } catch (InterruptedException e) {
            delegCertFile.delete();
        } finally {
            if (readIn != null) {
                try {
                    readIn.close();
                } catch (IOException ioEx) {
                    logger.error(ioEx.getMessage(), ioEx);
                }
            }

            if (readErr != null) {
                try {
                    readErr.close();
                } catch (IOException ioEx) {
                    logger.error(ioEx.getMessage(), ioEx);
                }
            }

            try {
                proc.getOutputStream().close();
            } catch (IOException ioEx) {}
        }         

        try {            
            proc = Runtime.getRuntime().exec(gridProxyInit_CMD);
        } catch (IOException ioe) {
            logger.error(ioe.getMessage(), ioe);
        } finally {
            if (proc != null) {
                try {
                    proc.waitFor();
                } catch (InterruptedException e) {}
                
                try {
                    proc.getInputStream().close();
                } catch (IOException ioEx) {}

                try {
                    proc.getOutputStream().close();
                } catch (IOException ioEx) {}

                try {
                    proc.getErrorStream().close();
                } catch (IOException ioEx) {}
            }

            delegCertFile.delete();
        }
        
        String cert = CEUtils.readFile(proxyPath + ".lmt");
        
        delegCertFile = new File(proxyPath + ".lmt");
        delegCertFile.delete();
        
        return cert;
    }
    
    public void delegationProxyAdded(DelegationProxyEvent event) {
        logger.debug("Begin delegationProxyAdded");
        DelegationProxyInfo dpi = event.getDelegationProxyInfo();
        
        if (event == null || dpi == null || proxyDir == null) {
            return;
        }
        
        String limitedProxy = null;
        try {            
            limitedProxy = makeLimitedDelegationProxy(dpi);

            SimpleDateFormat dateFormat = new SimpleDateFormat();
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));

            logger.info("New delegation proxy created [delegationId=" + dpi.getId() + "; userId=" + dpi.getUserId() + "] valid from " +
                    dateFormat.format(dpi.getStartTime()) + " (GMT) to " + dateFormat.format(dpi.getExpirationTime()) + " (GMT)");
        } catch (Throwable e) {
            logger.error("delegationProxyAdded error: cannot store the delegation proxy locally - " + e.getMessage());
        }
        
        if (logger.isDebugEnabled()) {
            logger.debug("DelegationProxyInfo = "+ dpi.toString());
            logger.debug("DelegationProxyId = " + dpi.getId());
            logger.debug("LocalUser = " + dpi.getLocalUser());
            logger.debug("LocalUserGroup = " + dpi.getLocalUserGroup());
            logger.debug("UserId = " + dpi.getUserId());
        }
        
        try {           
            CopyNewProxyToSandboxCmd cmd = new CopyNewProxyToSandboxCmd();
            cmd.setUserId(dpi.getUserId());
            cmd.setLocalUser(dpi.getLocalUser());
            cmd.setLocalUserGroup(dpi.getLocalUserGroup());
            cmd.setDelegationProxyId(dpi.getId());
            cmd.setDelegationProxy(limitedProxy);
            cmd.setDelegationProxySuffix(delegationSuffix);
            cmd.setCommandGroupId(dpi.getUserId());
            
            CommandManager.getInstance().insertCommand(cmd);
            logger.debug("insertCommand CopyNewProxyToSandboxCmd: done!");
            
            logger.debug("added new delegation proxy certificate [delegId=" + dpi.getId() + "] by user " + cmd.getUserId());
        } catch (CommandException e) {
            logger.error("delegationProxyAdded error: CommandException: " + e.getMessage());
        } catch (CommandManagementException e) {
            logger.error("delegationProxyAdded error: CommandManagementException: " + e.getMessage());
        }
        
        logger.debug("End delegationProxyAdded");
    }

    public void delegationProxyRemoved(DelegationProxyEvent event) {
        logger.debug("Begin delegationProxyRemoved");
        DelegationProxyInfo dpi = event.getDelegationProxyInfo();
        
        if (event == null || dpi == null || proxyDir == null) {
            return;
        }

        try {
            DeleteProxyFromSandboxCmd cmd = new DeleteProxyFromSandboxCmd();
            cmd.setUserId(dpi.getUserId());
            cmd.setUserDN(dpi.getDN());
            cmd.setUserFQAN(dpi.getFQAN());
            cmd.setLocalUser(dpi.getLocalUser());
            cmd.setLocalUserGroup(dpi.getLocalUserGroup());
            cmd.setDelegationProxyId(dpi.getId());
            cmd.setDelegationProxySuffix(delegationSuffix);
            cmd.setCommandGroupId(dpi.getUserId());

            CommandManager.getInstance().insertCommand(cmd);

            logger.debug("removed delegation proxy certificate [delegId=" + dpi.getId() + "] by user " + cmd.getUserId());
        } catch (CommandException e) {
            logger.error("delegationProxyRemoved error: CommandException: " + e.getMessage());
            return;
        } catch (CommandManagementException e) {
            logger.error("delegationProxyRemoved error: CommandManagementException: " + e.getMessage());
            return;
        }
        
        logger.info("Removed delegation proxy [delegationId=" + dpi.getId() + "; userId=" + dpi.getUserId());
        logger.debug("End delegationProxyRemoved");
    }

    public void delegationProxyUpdated(DelegationProxyEvent event) {
        logger.debug("Begin delegationProxyUpdated");
        DelegationProxyInfo dpi = event.getDelegationProxyInfo();
        
        if (event == null || dpi == null || proxyDir == null) {
            return;
        }
        
        String limitedProxy = null;
        try {            
            limitedProxy = makeLimitedDelegationProxy(dpi);
            
            SimpleDateFormat dateFormat = new SimpleDateFormat();
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));

            logger.info("Renewed delegation proxy [delegationId=" + dpi.getId() + "; userId=" + dpi.getUserId() + "] valid from " +
                    dateFormat.format(dpi.getStartTime()) + " (GMT) to " + dateFormat.format(dpi.getExpirationTime()) + " (GMT)");
        } catch (Exception e) { 
            logger.error("delegationProxyUpdated error: cannot store the delegation proxy locally - " + e.getMessage());         
            return;
        }
        
        try {
            UpdateProxyToSandboxCmd cmd = new UpdateProxyToSandboxCmd();
            cmd.setUserId(dpi.getUserId());
            cmd.setLocalUser(dpi.getLocalUser());
            cmd.setLocalUserGroup(dpi.getLocalUserGroup());
            cmd.setDelegationProxyId(dpi.getId());
            cmd.setDelegationProxy(limitedProxy);
            cmd.setDelegationProxySuffix(delegationSuffix);
            cmd.setDelegationProxyInfo(dpi.toString());
            cmd.setCommandGroupId(dpi.getUserId());
            
            CommandManager.getInstance().insertCommand(cmd);

            logger.debug("updated delegation proxy certificate [delegId=" + dpi.getId() + "] by user " + cmd.getUserId());
        } catch (CommandException e) {
            logger.error("delegationProxyAdded error: CommandException: " + e.getMessage());
            return;
        } catch (CommandManagementException e) {
            logger.error("delegationProxyAdded error: CommandManagementException: " + e.getMessage());
            return;
        }

        logger.debug("End delegationProxyUpdated");
    }
    
    public long getDbID() {
    	return dbID;
    }
    
    protected static void destroy() {
        logger.info("destroy invoked!");
        delegationService = null;
        serviceInfo = null;
        
        try {
            CommandManager.getInstance().destroy();
        } catch (CommandManagementException e) {
            logger.error(e.getMessage());
        }

        logger.info("destroyed!");
    }

    public QueryEventResult queryEvent(QueryEventRequest request) throws RemoteException, GenericFault, AuthorizationFault, InvalidArgumentFault {
        logger.debug("Begin queryEvent");
        
        if (request == null) {
            throw (new IllegalArgumentException("QueryEventRequest not specified!"));
        }
        
        if (!request.getEventType().equalsIgnoreCase("JOB_STATUS")) {
            throw FaultFactory.makeInvalidArgumentFault("queryEvent", "0", "event type \"" + request.getEventType() + "\" not supported!", null);
        }
       
//      logger.info(org.apache.axis.utils.XMLUtils.ElementToString(MessageContext.getCurrentContext().getRequestMessage().getSOAPEnvelope()));

        long startTime = Calendar.getInstance().getTimeInMillis();
        
        QueryEventResult result = new QueryEventResult();
        
        QueryEventCmd queryEventCmd = new QueryEventCmd(request.getEventType());
        queryEventCmd.setIsAdmin(false);
        queryEventCmd.setFromDate(request.getFromDate());
        queryEventCmd.setToDate(request.getToDate());
        queryEventCmd.setFromEventId(request.getFromEventId());
        queryEventCmd.setToEventId(request.getToEventId());
        queryEventCmd.setMaxQueryEventResultSize(request.getMaxQueryEventResultSize());
        
        boolean isIceRequest = false;        
        SOAPHeaderElement headerElement = MessageContext.getCurrentContext().getRequestMessage().getSOAPEnvelope().getHeaderByName("http://glite.org/2007/11/ce/cream/types", "iceId");

        if (headerElement == null) {
            headerElement = MessageContext.getCurrentContext().getRequestMessage().getSOAPEnvelope().getHeaderByName("http://glite.org/ce/creamapij/types", "iceId");
        }
        
        if (headerElement != null && headerElement.getValue() != null && headerElement.getValue().length() > 0) {
            queryEventCmd.setUserId(getUserDN_FQAN(null) + "@" + headerElement.getValue());

            headerElement.setProcessed(true);
            isIceRequest = true;
        } else {
           queryEventCmd.setUserId(getUserDN_FQAN(null));
        }

        org.glite.ce.creamapi.ws.cream2.types.Property[] requestProperty = request.getProperty();

        if (requestProperty != null && requestProperty.length > 0) {
            List<String> statusList = new ArrayList<String>(0);
            
            for (int i=0; i<requestProperty.length; i++) {                
                if ("status".equalsIgnoreCase(requestProperty[i].getName())) {
                    statusList.add(requestProperty[i].getValue());
                }
            }
            
            queryEventCmd.addParameter("statusList", statusList);
        } else if (isIceRequest) {
            List<String> statusList = new ArrayList<String>(9);
            statusList.add(JobStatus.getNameByType(JobStatus.ABORTED));
            statusList.add(JobStatus.getNameByType(JobStatus.CANCELLED));
            statusList.add(JobStatus.getNameByType(JobStatus.DONE_FAILED));
            statusList.add(JobStatus.getNameByType(JobStatus.DONE_OK));
            statusList.add(JobStatus.getNameByType(JobStatus.HELD));
            statusList.add(JobStatus.getNameByType(JobStatus.REALLY_RUNNING));
            statusList.add(JobStatus.getNameByType(JobStatus.RUNNING));
            
            queryEventCmd.addParameter("statusList", statusList);            
        }
        
        try {
            CommandManager.getInstance().insertCommand(queryEventCmd);

            List<Event> eventList = queryEventCmd.geEvents();

            org.glite.ce.creamapi.ws.cream2.types.Event[] eventArray = new org.glite.ce.creamapi.ws.cream2.types.Event[eventList.size()];

            Event event = null;

            for (int i = 0; i < eventList.size(); i++) {
                event = eventList.get(i);

                eventArray[i] = new org.glite.ce.creamapi.ws.cream2.types.Event();
                eventArray[i].setId(event.getId());
                eventArray[i].setTimestamp(event.getTimestamp());
                eventArray[i].setType(event.getType());

                if (logger.isDebugEnabled()) {
                    logger.debug("event [id=" + event.getId() + "; type=" + event.getType() + "; timestamp=" + event.getTimestamp().getTime() + "]");
                }

                List<org.glite.ce.creamapi.eventmanagement.Property> eventPropertyList = event.getProperty();
                org.glite.ce.creamapi.ws.cream2.types.Property[] eventPropertyArray = new org.glite.ce.creamapi.ws.cream2.types.Property[eventPropertyList.size()];
                String value = null;

                for (int x = 0; x < eventPropertyList.size(); x++) {
                    value = (String)eventPropertyList.get(x).getValue();
                    value = value == null? "N/A" : value;
                   
                    eventPropertyArray[x] = new org.glite.ce.creamapi.ws.cream2.types.Property(eventPropertyList.get(x).getName(), value);
                    logger.debug("event property name=" + eventPropertyArray[x].getName() + " value=" + eventPropertyArray[x].getValue());

                }
 
                eventArray[i].setProperty(eventPropertyArray);
            }
            
            result.setEvent(eventArray);
        } catch (IllegalArgumentException e) {
            logger.error("IllegalArgumentException: " + e.getMessage());
            throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
        } catch (CommandException e) {
            logger.error("CommandException: " + e.getMessage());
            throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
        } catch (CommandManagementException e) {
            logger.error("CommandManagementException: " + e.getMessage());
            throw FaultFactory.makeGenericFault("insertCommand", "0", e.getMessage(), e.getMessage());
        }
        
        result.setDbId(""+dbID);
        result.setQueryExecutionTimeInMillis(Long.valueOf(Calendar.getInstance().getTimeInMillis() - startTime));
        
        logger.debug("End queryEvent");
        
        return result;
    }
}
