/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream.ws.authz;

import javax.xml.namespace.QName;

import org.glite.authz.pep.profile.GridCEAuthorizationProfile;
import org.glite.ce.commonj.authz.AuthorizationHandler;
import org.glite.ce.commonj.authz.argus.ActionMappingInterface;

public class ActionMapping
    implements ActionMappingInterface {

    public final static String XACML_PREFIX = "http://glite.org/xacml/";

    public static final String ID_ATTRIBUTE_JOB_ADMIN = XACML_PREFIX + "obligation/local-role/administrator";

    public static final String[] mandProperties = { AuthorizationHandler.LOCAL_USER_ID,
            AuthorizationHandler.LOCAL_GROUP_ID };

    public String getXACMLAction(QName operation) {

        String opName = operation.getLocalPart();

        if (opName.equals("JobRegister") || opName.equals("JobStart") || opName.equals("JobSetLeaseId")) {
            return GridCEAuthorizationProfile.ACTION_JOB_SUBMIT;
        }

        if (opName.equals("JobCancel") || opName.equals("JobPurge")) {
            return GridCEAuthorizationProfile.ACTION_JOB_TERMINATE;
        }

        if (opName.equals("JobInfo") || opName.equals("JobStatus") || opName.equals("JobList")
                || opName.equals("QueryEvent")) {
            return GridCEAuthorizationProfile.ACTION_JOB_GET_INFO;
        }

        if (opName.equals("JobSuspend") || opName.equals("JobResume")) {
            return GridCEAuthorizationProfile.ACTION_JOB_MANAGE;
        }

        if (opName.equals("getLease") || opName.equals("getLeaseList")) {
            return GridCEAuthorizationProfile.ACTION_LEASE_GET_INFO;
        }

        if (opName.equals("setLease") || opName.equals("deleteLease")) {
            return GridCEAuthorizationProfile.ACTION_LEASE_MANAGE;
        }

        if (opName.equals("getServiceInfo") || opName.equals("getVersion") || opName.equals("getInterfaceVersion")
                || opName.equals("getServiceMetadata")) {
            return GridCEAuthorizationProfile.ACTION_GET_INFO;
        }

        if (opName.equals("getTerminationTime")) {
            return GridCEAuthorizationProfile.ACTION_DELEGATION_GET_INFO;
        }

        if (opName.equals("getProxyReq") || opName.equals("getNewProxyReq") || opName.equals("putProxy")
                || opName.equals("renewProxyReq") || opName.equals("destroy")) {
            return GridCEAuthorizationProfile.ACTION_DELEGATION_MANAGE;
        }

        return null;
    }

    public String getAttributeMapping(String obId, String attrId) {
        if (GridCEAuthorizationProfile.ID_OBLIGATION_POSIX_ENV_MAP.equals(obId)) {
            if (GridCEAuthorizationProfile.ID_ATTRIBUTE_USER_ID.equals(attrId)) {
                return AuthorizationHandler.LOCAL_USER_ID;
            }

            if (GridCEAuthorizationProfile.ID_ATTRIBUTE_PRIMARY_GROUP_ID.equals(attrId)) {
                return AuthorizationHandler.LOCAL_GROUP_ID;
            }
        }

        if (ID_ATTRIBUTE_JOB_ADMIN.equals(obId)) {
            return AuthorizationHandler.JOB_ADMIN;
        }
        return null;
    }

    public String[] getMandatoryProperties() {
        return mandProperties;
    }
}
