/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.cream.ws.authz;

import java.util.Calendar;

import javax.xml.namespace.QName;

//import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.description.OperationDesc;
import org.glite.ce.creamapi.ws.cream2.types.AuthorizationFault;
import org.glite.security.delegation.DelegationException;

public class AdminHandler
    extends org.glite.ce.commonj.authz.AdminHandler {

    static final long serialVersionUID = 1297346792;

    private static String DELEGATION_NS = "http://www.gridsite.org/namespaces/delegation-2";

    protected Object getAuthorizationFault(String message) {

        AuthorizationFault fault = new AuthorizationFault();
        fault.setMethodName("invoke");
        fault.setDescription(message);
        fault.setErrorCode("0");
        fault.setFaultCause(message);
        fault.setTimestamp(Calendar.getInstance());

        MessageContext msgContext = MessageContext.getCurrentContext();
        OperationDesc operation = msgContext.getOperation();
        if (operation == null) {
            return fault;
        }
        QName qName = operation.getElementQName();
        if (qName != null && qName.getNamespaceURI().equals(DELEGATION_NS)) {
            return new DelegationException(message);
        }

        return fault;
    }

}
