/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando <zangrando@pd.infn.it>
 *          Paolo Andreetto <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.cream.ws.delegation.storage;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.commonj.utils.CEUtils;
import org.glite.ce.cream.ws.delegation.DelegationProxyEvent;
import org.glite.ce.cream.ws.delegation.DelegationProxyInfo;
import org.glite.ce.cream.ws.delegation.DelegationProxyListener;
import org.glite.security.delegation.GrDProxyDlgeeOptions;
import org.glite.security.delegation.storage.GrDPStorageCacheElement;
import org.glite.security.delegation.storage.GrDPStorageDatabase;
import org.glite.security.delegation.storage.GrDPStorageElement;
import org.glite.security.delegation.storage.GrDPStorageException;
import org.glite.security.util.DNHandler;

public class CREAMGrDPDatabaseStorage extends GrDPStorageDatabase implements CREAMGrDPStorageInterface {
    private final static Logger logger = Logger.getLogger(CREAMGrDPDatabaseStorage.class.getName());
//    private static Timer timer = null;

    public CREAMGrDPDatabaseStorage(GrDProxyDlgeeOptions dlgeeOpt) throws GrDPStorageException {
        super(dlgeeOpt);
        logger.debug("Entered in CREAMGrDPDatabaseStorage dlgeeOpt.getDlgeeStorageFactory() = " + dlgeeOpt.getDlgeeStorageFactory());

//        ServiceConfig serviceConf = ServiceConfig.getConfiguration();
//
//        int purgerRateInMinutes = 720;
//
//        try {
//            purgerRateInMinutes = Integer.parseInt(serviceConf.getStringParameter(ServiceAttribute.DELEGATION_PURGE_RATE_LABEL));
//        } catch (Throwable t) {
//            logger.warn("Configuration warning: wrong value for " + ServiceAttribute.DELEGATION_STORAGE_LABEL + " property => using default " + purgerRateInMinutes);
//        }
//
//        if (purgerRateInMinutes == -1) {
//            logger.info("the delegation proxy purger is disabled!");
//        } else {
//            if (timer == null) {
//                Calendar time = Calendar.getInstance();
//                time.add(Calendar.SECOND, 10);
//
//                timer = new Timer("TIMER", true);
//                timer.schedule(new DelegationProxyPurger(this), time.getTime(), purgerRateInMinutes * 60000);
//            }
//        }
    }

    public void addDelegationProxyListener(DelegationProxyListener l) {
        EventDispatcher.getInstance().addDelegationProxyListener(l);
    }

    public void deleteGrDPStorageElement(String delegationId, String dn, String localUser) throws GrDPStorageException {
        logger.debug("Begin deleteGrDPStorageElement for delegationId = " + delegationId + " dn=" + dn);
        deleteDelegationProxy(delegationId, dn, localUser);
        logger.debug("Begin deleteGrDPStorageElement for delegationId = " + delegationId + " dn=" + dn);
    }

    public void destroy() {
//        if (timer != null) {
//            timer.cancel();
//            timer = null;
//        }
    }

    public GrDPStorageCacheElement findGrDPStorageCacheElement(String delegationId, String dn) throws GrDPStorageException {
        if (delegationId == null) {
            throw new GrDPStorageException("delegationId not specified!");
        }
        if (dn == null) {
            throw new GrDPStorageException("DN not specified!");
        }

        try {
            return super.findGrDPStorageCacheElement(delegationId, dn);
        } catch (NullPointerException e) {
            logger.error("findGrDPStorageCacheElement error: " + e.getMessage());
            throw new GrDPStorageException("database connection problem");
        } catch (Throwable e) {
            logger.error("findGrDPStorageCacheElement error: " + e.getMessage());
            throw new GrDPStorageException("findGrDPStorageCacheElement error: " + e.getMessage());
        }
    }

    public DelegationProxyListener[] getDelegationProxyListeners() {
        return EventDispatcher.getInstance().getDelegationProxyListeners();
    }

    public void insertGrDPStorageElement(GrDPStorageElement elem) throws GrDPStorageException {
        logger.debug("Begin insertGrDPStorageElement");
        if (elem == null) {
            logger.error("GrDPStorageElement not specified!");
            throw new GrDPStorageException("GrDPStorageElement not specified!");
        }

        DelegationProxyInfo delegProxyInfo = null;

        try {
            delegProxyInfo = DelegationProxyInfo.getInstance(elem.getDelegationID(), elem.getCertificate());
            delegProxyInfo.setLocalUser(CEUtils.getLocalUser());
            delegProxyInfo.setLocalUserGroup(CEUtils.getLocalUserGroup());
        } catch (Throwable e) {
            logger.error("insertGrDPStorageElement error: " + e.getMessage());
            throw new GrDPStorageException("insertGrDPStorageElement error: " + e.getMessage());
        }

        insertDelegationProxy(delegProxyInfo);

        logger.debug("End insertGrDPStorageElement");
    }

    public void removeDelegationProxyListener(DelegationProxyListener l) {
        logger.debug("Begin removeDelegationProxyListener");
        EventDispatcher.getInstance().removeDelegationProxyListener(l);
        logger.debug("End removeDelegationProxyListener");
    }
    
    /*
     * (non-Javadoc)
     * 
     * @seeorg.glite.ce.cream.ws.delegation.storage.CREAMGrDPStorageInterface#
     * updateGrDPStorageElement
     * (org.glite.security.delegation.storage.GrDPStorageElement)
     */
    public void updateGrDPStorageElement(GrDPStorageElement elem) throws GrDPStorageException {
        logger.debug("Begin updateGrDPStorageElement");

        if (elem == null) {
            throw new GrDPStorageException("GrDPStorageElement not specified!");
        }

        DelegationProxyInfo delegProxyInfo = null;

        try {
            delegProxyInfo = DelegationProxyInfo.getInstance(elem.getDelegationID(), elem.getCertificate());
            delegProxyInfo.setLocalUser(CEUtils.getLocalUser());
            delegProxyInfo.setLocalUserGroup(CEUtils.getLocalUserGroup());
        } catch (Throwable e) {
            logger.error("updateGrDPStorageElement error: " + e.getMessage());
            throw new GrDPStorageException("updateGrDPStorageElement error: " + e.getMessage());
        }

        updateDelegationProxy(delegProxyInfo);

        logger.debug("End updateGrDPStorageElement");
    }

    public void deleteDelegationProxy(String delegationId, String dn, String localUser) throws GrDPStorageException {
        logger.debug("Begin deleteDelegationProxy for delegationId = " + delegationId + " dn=" + dn);

        if (delegationId == null || delegationId.length() == 0) {
            throw new GrDPStorageException("delegationId not specified!");
        }

        if (dn == null || dn.length() == 0) {
            throw new GrDPStorageException("user DN not specified!");
        }
        
        if (localUser == null || localUser.length() == 0) {
            throw new GrDPStorageException("localUser not specified!");
        }

        try {
            DelegationProxyInfo proxy = getDelegationProxy(delegationId, dn, localUser);

            if (proxy != null) {
                Connection connection = null;

                try {
                    connection = DatasourceManager.getConnection("java:comp/env/jdbc/dlg");
                } catch (DatabaseException e) {
                    throw new GrDPStorageException("getConnection error: " + e.getMessage());
                }

                PreparedStatement deletePreparedStatement = null;
                try {
                    deletePreparedStatement = connection.prepareStatement("delete from t_credential where dlg_id = \'" + delegationId + "\' and dn=\'" + DNHandler.getDN(dn).getX500() + "\'");
                    deletePreparedStatement.executeUpdate();

                    connection.commit();
                } catch (SQLException sqle) {
                    connection.rollback();
                    throw new GrDPStorageException("Rollback executed due to: " + sqle.getMessage());
                } finally {
                    if (deletePreparedStatement != null) {
                        try {
                            deletePreparedStatement.close();
                        } catch (SQLException sqle1) {
                            logger.error(sqle1);
                        }
                    }

                    if (connection != null) {
                        try {
                            connection.close();
                        } catch (SQLException e) {
                            logger.error(e);
                        }
                    }
                }

                super.deleteGrDPStorageCacheElement(delegationId, dn);

                logger.debug("Calling fireDelegationProxyEvent for DELEGATIONPROXY_REMOVED");
                EventDispatcher.getInstance().fireDelegationProxyEvent(new DelegationProxyEvent(DelegationProxyEvent.DELEGATIONPROXY_REMOVED, proxy));
                logger.debug("fireDelegationProxyEvent for DELEGATIONPROXY_REMOVED: done!");
            }
        } catch (NullPointerException e) {
            throw new GrDPStorageException("database connection problem");
        } catch (Throwable e) {
            throw new GrDPStorageException("deleteGrDPStorageElement error: " + e.getMessage());
        }
        logger.debug("End deleteDelegationProxy for delegationId = " + delegationId + " dn=" + dn);
    }

    public List<DelegationProxyInfo> findExpiredDelegationProxy() throws GrDPStorageException {
        logger.debug("Begin findExpiredDelegationProxy");
        Connection connection = null;

        try {
            connection = DatasourceManager.getConnection("java:comp/env/jdbc/dlg");
        } catch (DatabaseException e) {
            throw new GrDPStorageException(e.getMessage());
        }

        List<DelegationProxyInfo> result = new ArrayList<DelegationProxyInfo>(0);

        PreparedStatement selectPreparedStatement = null;
        try {
            selectPreparedStatement = connection.prepareStatement("select dlg_id, dn, local_user from t_credential where termination_time < ?");
            selectPreparedStatement.setTimestamp(1, new Timestamp(Calendar.getInstance().getTimeInMillis()));

            DelegationProxyInfo proxy = null;

            // execute query, and return number of rows created
            ResultSet resultSet = selectPreparedStatement.executeQuery();
            while (resultSet.next()) {
                proxy = getDelegationProxy(resultSet.getString("dlg_id"), resultSet.getString("dn"), resultSet.getString("local_user"));

                if (proxy != null) {
                    result.add(proxy);
                }
            }
        } catch (SQLException sqle) {
            throw new GrDPStorageException(sqle.getMessage());
        } finally {
            if (selectPreparedStatement != null) {
                try {
                    selectPreparedStatement.close();
                } catch (SQLException sqle1) {
                    logger.error(sqle1);
                }
            }

            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }
        }
        logger.debug("End findExpiredDelegationProxy");
        return result;
    }

    public DelegationProxyInfo getDelegationProxy(String delegationId, String dn, String localUser) throws GrDPStorageException {
        if (delegationId == null || delegationId.length() == 0) {
            throw new GrDPStorageException("delegationId not specified!");
        }

        if (dn == null || dn.length() == 0) {
            throw new GrDPStorageException("user DN not specified!");
        }
        
        if (localUser == null || localUser.length() == 0) {
            throw new GrDPStorageException("localUser not specified!");
        }
        
        logger.debug("Begin getDelegationProxy for delegationId = " + delegationId + " dn=" + dn + " localUser=" + localUser);

        Connection connection = null;
        DelegationProxyInfo proxy = null;

        try {
            connection = DatasourceManager.getConnection("java:comp/env/jdbc/dlg");
        } catch (DatabaseException e) {
            throw new GrDPStorageException("getConnection error: " + e.getMessage());
        }

        String sql = "SELECT dlg_id, dn, fqan, vo, local_user, local_user_group, proxy, info, voms_attrs, termination_time, last_update_time, start_time FROM t_credential WHERE dlg_id = ? AND dn = ? AND local_user = ?";
        PreparedStatement getProxyStatement = null;
        ResultSet rs = null;

        try {
            getProxyStatement = connection.prepareStatement(sql);
            getProxyStatement.setString(1, delegationId);
            getProxyStatement.setString(2, DNHandler.getDN(dn).getX500());
            getProxyStatement.setString(3, localUser);

            rs = getProxyStatement.executeQuery();

            if (rs.next()) {
                StringTokenizer st = new StringTokenizer("\t");
                ArrayList<String> vomsAttributes = new ArrayList<String>(0);

                while (st.hasMoreTokens()) {
                    vomsAttributes.add(st.nextToken());
                }

                proxy = DelegationProxyInfo.getInstance(rs.getString("dlg_id"));
                proxy.setDN(rs.getString("dn"));
                proxy.setFQAN(rs.getString("fqan"));
                proxy.setVO(rs.getString("vo"));
                proxy.setLocalUser(rs.getString("local_user"));
                proxy.setLocalUserGroup(rs.getString("local_user_group"));
                proxy.setProxy(rs.getString("proxy"));
                proxy.setInfo(rs.getString("info"));
                proxy.setExpirationTime(rs.getTimestamp("termination_time"));
                proxy.setLastUpdateTime(rs.getTimestamp("last_update_time"));
                proxy.setStartTime(rs.getTimestamp("start_time"));
                proxy.setVomsAttributes((String[]) vomsAttributes.toArray(new String[] {}));
            }
        } catch (SQLException e) {
            logger.error("Failure on db interaction.", e);
            try {
                connection.rollback();
            } catch (SQLException sqle) {
                logger.error(sqle);
            }
            throw new GrDPStorageException("Internal failure.");
        } finally {
            if (getProxyStatement != null) {
                try {
                    getProxyStatement.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }

            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }

            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }
        }

        return proxy;
    }

    public void insertDelegationProxy(DelegationProxyInfo proxy) throws GrDPStorageException {
        if (proxy == null) {
            throw new GrDPStorageException("updateDelegationProxy not specified!");
        }

        logger.debug("Begin insertDelegationProxy for delegationId = " + proxy.getId() + " dn=" + proxy.getDN());

        if (proxy.getLocalUser() == null) {
            proxy.setLocalUser(CEUtils.getLocalUser());
        }

        if (proxy.getLocalUserGroup() == null) {
            proxy.setLocalUserGroup(CEUtils.getLocalUserGroup());
        }

        Connection connection = null;

        try {
            connection = DatasourceManager.getConnection("java:comp/env/jdbc/dlg");
        } catch (DatabaseException e) {
            throw new GrDPStorageException("getConnection error: " + e.getMessage());
        }

        String sql = "INSERT INTO t_credential (dlg_id, dn, fqan, vo, local_user, local_user_group, proxy, info, voms_attrs, termination_time, last_update_time, start_time) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
        PreparedStatement insertStatement = null;

        try {
            String vomsAttrsStr = "";
            for (int i = 0; i < proxy.getVomsAttributes().length; i++) {
                vomsAttrsStr += "\t" + proxy.getVomsAttributes()[i];
            }

            insertStatement = connection.prepareStatement(sql);
            insertStatement.setString(1, proxy.getId());
            insertStatement.setString(2, DNHandler.getDN(proxy.getDN()).getX500());
            insertStatement.setString(3, proxy.getFQAN());
            insertStatement.setString(4, proxy.getVO());
            insertStatement.setString(5, proxy.getLocalUser());
            insertStatement.setString(6, proxy.getLocalUserGroup());
            insertStatement.setString(7, proxy.getProxy());
            insertStatement.setString(8, proxy.getInfo());
            insertStatement.setString(9, vomsAttrsStr);

            if (proxy.getExpirationTime() != null) {
                insertStatement.setTimestamp(10, new java.sql.Timestamp(proxy.getExpirationTime().getTime()));
            } else {
                insertStatement.setTimestamp(10, null);
            }

            if (proxy.getLastUpdateTime() != null) {
                insertStatement.setTimestamp(11, new java.sql.Timestamp(proxy.getLastUpdateTime().getTime()));
            } else {
                insertStatement.setTimestamp(11, null);
            }

            if (proxy.getStartTime() != null) {
                insertStatement.setTimestamp(12, new java.sql.Timestamp(proxy.getStartTime().getTime()));
            } else {
                insertStatement.setTimestamp(12, null);
            }

            insertStatement.executeUpdate();

            // Commit
            connection.commit();
        } catch (SQLException e) {
            logger.error("Failure on db interaction.", e);
            try {
                connection.rollback();
            } catch (SQLException sqle) {
                logger.error(sqle);
            }
            throw new GrDPStorageException("Internal failure.");
        } finally {
            if (insertStatement != null) {
                try {
                    insertStatement.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }

            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }
        }

        try {
            logger.debug("Calling fireDelegationProxyEvent for DELEGATIONPROXY_ADDED with CEUtils.getLocalUser()=" + CEUtils.getLocalUser() + " CEUtils.getLocalUserGroup()=" + CEUtils.getLocalUserGroup());
            EventDispatcher.getInstance().fireDelegationProxyEvent(new DelegationProxyEvent(DelegationProxyEvent.DELEGATIONPROXY_ADDED, proxy));
            logger.debug("fireDelegationProxyEvent for DELEGATIONPROXY_ADDED with CEUtils.getLocalUser()=" + CEUtils.getLocalUser() + " CEUtils.getLocalUserGroup()=" + CEUtils.getLocalUserGroup() + ": done!");
        } catch (Throwable e) {
            logger.error("insertGrDPStorageElement error: " + e.getMessage());
            throw new GrDPStorageException("insertGrDPStorageElement error: " + e.getMessage());
        }

        logger.debug("End insertDelegationProxy for delegationId = " + proxy.getId() + " dn=" + proxy.getDN());
    }

    public void updateDelegationProxy(DelegationProxyInfo proxy) throws GrDPStorageException {
        if (proxy == null) {
            throw new GrDPStorageException("updateDelegationProxy not specified!");
        }

        proxy.setLastUpdateTime(Calendar.getInstance().getTime());

        if (proxy.getLocalUser() == null) {
            proxy.setLocalUser(CEUtils.getLocalUser());
        }

        if (proxy.getLocalUserGroup() == null) {
            proxy.setLocalUserGroup(CEUtils.getLocalUserGroup());
        }

        logger.debug("Begin updateDelegationProxy for delegationId = " + proxy.getId() + " dn=" + proxy.getDN());

        Connection connection = null;

        try {
            connection = DatasourceManager.getConnection("java:comp/env/jdbc/dlg");
        } catch (DatabaseException e) {
            throw new GrDPStorageException("getConnection error: " + e.getMessage());
        }

        String sql = "UPDATE t_credential C SET proxy = ?, info = ?, voms_attrs = ?, termination_time = ?, last_update_time = ?, start_time = ? WHERE dlg_id = ? AND dn = ?";
        PreparedStatement updateStatement = null;

        try {
            String vomsAttrsStr = "";
            for (int i = 0; i < proxy.getVomsAttributes().length; i++) {
                vomsAttrsStr += "\t" + proxy.getVomsAttributes()[i];
            }

            updateStatement = connection.prepareStatement(sql);
            updateStatement.setString(1, proxy.getProxy());
            updateStatement.setString(2, proxy.getInfo());
            updateStatement.setString(3, vomsAttrsStr);
            updateStatement.setTimestamp(4, new java.sql.Timestamp(proxy.getExpirationTime().getTime()));
            updateStatement.setTimestamp(5, new java.sql.Timestamp(proxy.getLastUpdateTime().getTime()));
            updateStatement.setTimestamp(6, new java.sql.Timestamp(proxy.getStartTime().getTime()));
            updateStatement.setString(7, proxy.getId());
            updateStatement.setString(8, DNHandler.getDN(proxy.getDN()).getX500());
            updateStatement.executeUpdate();

            // Commit
            connection.commit();
        } catch (SQLException e) {
            logger.error("Failure on db interaction.", e);
            try {
                connection.rollback();
            } catch (SQLException e1) {
                logger.error(e);
            }
            throw new GrDPStorageException("Internal failure.");
        } finally {
            if (updateStatement != null) {
                try {
                    updateStatement.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }

            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error(e);
                }
            }
        }

        try {
            logger.debug("Calling fireDelegationProxyEvent for DELEGATIONPROXY_UPDATED with CEUtils.getLocalUser()=" + CEUtils.getLocalUser() + " CEUtils.getLocalUserGroup()=" + CEUtils.getLocalUserGroup());
            EventDispatcher.getInstance().fireDelegationProxyEvent(new DelegationProxyEvent(DelegationProxyEvent.DELEGATIONPROXY_UPDATED, proxy));
            logger.debug("fireDelegationProxyEvent for DELEGATIONPROXY_UPDATED with CEUtils.getLocalUser()=" + CEUtils.getLocalUser() + " CEUtils.getLocalUserGroup()=" + CEUtils.getLocalUserGroup() + ": done!");
        } catch (Throwable e) {
            logger.error("updateDelegationProxy error: " + e.getMessage());
            throw new GrDPStorageException("updateGrDPStorageElement error: " + e.getMessage());
        }

        logger.debug("End updateDelegationProxy for delegationId = " + proxy.getId() + " dn=" + proxy.getDN());
    }
}
