/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Luigi Zangrando <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.cream.ws.delegation.storage;

import java.util.List;
import java.util.TimerTask;

import org.apache.log4j.Logger;
import org.glite.ce.cream.ws.delegation.DelegationProxyInfo;
import org.glite.security.delegation.storage.GrDPStorageException;

public class DelegationProxyPurger extends TimerTask {
    private final static Logger logger = Logger.getLogger(DelegationProxyPurger.class.getName());
    private CREAMGrDPDatabaseStorage grDPStorage = null;

    public DelegationProxyPurger(CREAMGrDPDatabaseStorage grDPStorage) {
        this.grDPStorage = grDPStorage;
    }

    public void run() {
        logger.info("BEGIN DelegationProxyPurger");
        try {
            List<DelegationProxyInfo> expiredDelegationList = grDPStorage.findExpiredDelegationProxy();

            for (DelegationProxyInfo delegation : expiredDelegationList) {
                logger.debug("deleting expired delegation [delegationId=" + delegation.getId() + "] [DN=" + delegation.getDN() + "] [localUser=" + delegation.getLocalUser() + "] [ExpirationTime=" + delegation.getExpirationTime() + "]");

                grDPStorage.deleteDelegationProxy(delegation.getId(), delegation.getDN(), delegation.getLocalUser());                

                logger.info("deleted expired delegation [delegationId=" + delegation.getId() + "] [DN=" + delegation.getDN() + "] [localUser=" + delegation.getLocalUser() + "] [ExpirationTime=" + delegation.getExpirationTime() + "]");
            }
        } catch (GrDPStorageException e) {
            logger.error(e.getMessage());
        } finally {
            logger.info("END DelegationProxyPurger");
        }
    }
}
