/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream.ws.utils;

import java.util.Calendar;

import org.glite.ce.creamapi.ws.cream2.types.AuthorizationFault;
import org.glite.ce.creamapi.ws.cream2.types.DateMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.DelegationIdMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.DelegationProxyFault;
import org.glite.ce.creamapi.ws.cream2.types.GenericFault;
import org.glite.ce.creamapi.ws.cream2.types.InvalidArgumentFault;
import org.glite.ce.creamapi.ws.cream2.types.JobStatusInvalidFault;
import org.glite.ce.creamapi.ws.cream2.types.JobSubmissionDisabledFault;
import org.glite.ce.creamapi.ws.cream2.types.JobUnknownFault;
import org.glite.ce.creamapi.ws.cream2.types.LeaseIdMismatchFault;
import org.glite.ce.creamapi.ws.cream2.types.NoSuitableResourcesFault;
import org.glite.ce.creamapi.ws.cream2.types.OperationNotSupportedFault;

public class FaultFactory {

    public static AuthorizationFault makeAuthorizationFault(String methodName, String errorCode, String description, String faultCause) {
        AuthorizationFault fault = new AuthorizationFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static DateMismatchFault makeDateMismatchFault(String methodName, String errorCode, String description, String faultCause) {
        DateMismatchFault fault = new DateMismatchFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static DelegationIdMismatchFault makeDelegationIdMismatchFault(String methodName, String errorCode, String description, String faultCause) {
        DelegationIdMismatchFault fault = new DelegationIdMismatchFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }
    
    public static DelegationProxyFault makeDelegationProxyFault(String methodName, String errorCode, String description, String faultCause) {
        DelegationProxyFault fault = new DelegationProxyFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }
    
    public static GenericFault makeGenericFault(String methodName, String errorCode, String description, String faultCause) {
        GenericFault fault = new GenericFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }
    
    public static InvalidArgumentFault makeInvalidArgumentFault(String methodName, String errorCode, String description, String faultCause) {
        InvalidArgumentFault fault = new InvalidArgumentFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static JobStatusInvalidFault makeJobStatusInvalidFault(String methodName, String errorCode, String description, String faultCause) {
        JobStatusInvalidFault fault = new JobStatusInvalidFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static JobSubmissionDisabledFault makeJobSubmissionDisabledFault(String methodName, String errorCode, String description, String faultCause) {
        JobSubmissionDisabledFault fault = new JobSubmissionDisabledFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static JobUnknownFault makeJobUnknownFault(String methodName, String errorCode, String description, String faultCause) {
        JobUnknownFault fault = new JobUnknownFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static LeaseIdMismatchFault makeLeaseIdMismatchFault(String methodName, String errorCode, String description, String faultCause) {
    	LeaseIdMismatchFault fault = new LeaseIdMismatchFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static NoSuitableResourcesFault makeNoSuitableResourcesFault(String methodName, String errorCode, String description, String faultCause) {
        NoSuitableResourcesFault fault = new NoSuitableResourcesFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }

    public static OperationNotSupportedFault makeOperationNotSupportedFault(String methodName, String errorCode, String description, String faultCause) {
        OperationNotSupportedFault fault = new OperationNotSupportedFault();
        fault.setMethodName(methodName);
        fault.setErrorCode(errorCode);
        fault.setDescription(description);
        fault.setFaultCause(faultCause);
        fault.setTimestamp(Calendar.getInstance());

        return fault;
    }
}
